#region Header
//
// .NET Sample
//
// Copyright (C) 2005-2007 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software
// for any purpose and without fee is hereby granted, provided
// that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Header

#region Namespaces
using System;
using System.Windows.Forms;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.EditorInput;
#endregion // Namespaces

// Optional to optimize the loading/searching
// - otherwise searching the first IExtensionApplication
//
[assembly: ExtensionApplication( typeof( AcaNetIntroCs.App ) )]

// Optional to optimize the loading/searching, but then only the commands within these classes are recognized
// - otherwise searching all classes for CommandMethod
//
//[assembly: CommandClass( typeof( AcaNetIntroCs.TestCmds ) )]

namespace AcaNetIntroCs
{
  /// <summary>
  /// Optional extension application class providing initialisation and termination callback functionality.
  /// </summary>
  public class App : IExtensionApplication
  {
    const string _Title = "ACA.NET Intro";

    static public void MsgBox( string msg )
    {
      MessageBox.Show( msg, _Title ); // todo
    }

    public void Initialize()
    {
      Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
      ed.WriteMessage( "\nLoading " + _Title + "..." );
      ed.WriteMessage( " defining commands AppCmd, HelloAca, -HelloAca, TestStatic, TestCount, TestException, TestSelect, TestRotate, TestReflection, TestSettings, TestWallAdd, TestWallStyles, TestAnchor, TestHouse, TestProperties, TestDispProps, TestComDispProps" );
    }

    public void Terminate()
    {
      // Cleanup.
      // Note: no "NETUNLOAD" available (current Microsoft limitation)
    }

    /// <summary>
    /// Example of a command defined within the App class.
    /// </summary>
    [CommandMethod( "AppCmd" )]
    static public void AppCmd()
    {
      MsgBox( "This is App.AppCmd()." );
    }
  }

  /// <summary>
  /// A minimal command definition.
  /// </summary>
  public class HelloAca
  {
    [CommandMethod( "HelloAca" )]
    static public void HelloAcaDialog()
    {
      App.MsgBox( "Hello ACA.NET!" );
    }

    [CommandMethod( "-HelloAca" )]
    static public void HelloAcaCommandLine()
    {
      Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
      ed.WriteMessage( "Hello ACA.NET!" );
    }
  }

  /// <summary>
  /// A class containing commands.
  /// </summary>
  public class TestCmds
  {
    int _count1; // per-doc data
    static int _count2; // global data

    /// <summary>
    /// Constructor is called when the first non-static command is called in any given doc.
    /// </summary>
    public TestCmds()
    {
      App.MsgBox( "TestCmds constructor." );
      // Further initialize per-doc data if needed...
    }

    [CommandMethod( "TestStatic" )]
    static public void StaticCommand()
    {
      App.MsgBox( "Static Command - no constructor called." );
    }

    static string _pluralSuffix( int n )
    {
      return 1 == n ? "" : "s";
    }

    static string _nTimes( int n )
    {
      return n.ToString() + " time" + _pluralSuffix( n );
    }

    [CommandMethod( "TestCount" )]
    public void TestCount()
    {
      ++_count1;
      ++_count2;
      string msg = string.Format( "This command has been invoked {0} in this document and {1} in this session.", 
        _nTimes( _count1 ), _nTimes( _count2 ) );
      App.MsgBox( msg );
    }
  }
}
