#region Header
//
// .NET Sample
//
// Copyright (C) 2005-2007 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software
// for any purpose and without fee is hereby granted, provided
// that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Header

#region Namespaces
using System.Reflection;

// AutoCAD
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;

// ACA (Aec)
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.ApplicationServices;
using Autodesk.Aec.PropertyData;
using Autodesk.Aec.PropertyData.DatabaseServices;
using Autodesk.Aec.Arch.ApplicationServices;
using Autodesk.Aec.Arch.DatabaseServices;
using Autodesk.Aec.Arch.Geometry;

// To resolve Object/Entity ambiguities
using AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject;
using AecObject = Autodesk.Aec.DatabaseServices.DBObject;
using AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity;
using AecEntity = Autodesk.Aec.DatabaseServices.Entity;

// To resolve Application ambiguities
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;

// To resolve TransactionManager ambiguities
using DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager; //wraps AcDbTransactionManager
using AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager; //wraps AcTransactionManager

// To resolve ObjectId ambiguities
using ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId;
using ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection;

#endregion // Namespaces

namespace AcaNetIntroCs
{
  /// <summary>
  /// A class containing commands.
  /// </summary>
  public class Commands3
  {
    /// <summary>
    /// List all properties from all property sets attached to all entities.
    /// </summary>
    [CommandMethod( "TestProperties" )]
    static public void TestProperties()
    {
      try
      {
        Database db = HostApplicationServices.WorkingDatabase;
        Editor ed = AcadApp.DocumentManager.MdiActiveDocument.Editor;
        using( Transaction tr = db.TransactionManager.StartTransaction() )
        {
          // Loop all Entities
          BlockTable bt = tr.GetObject( db.BlockTableId, OpenMode.ForRead ) as BlockTable;
          BlockTableRecord btr = tr.GetObject( bt[BlockTableRecord.ModelSpace], OpenMode.ForRead ) as BlockTableRecord;
          foreach( ObjectId id in btr )
          {
            // get the property sets collection on this entity
            AcObject obj = tr.GetObject( id, OpenMode.ForRead );
            ObjectIdCollection idsPropSets = PropertyDataServices.GetPropertySets( obj );
            // how many property sets does it have?  
            ed.WriteMessage( "\n" + obj.GetRXClass().Name + " (" + obj.Handle.ToString() + ") has " 
              + idsPropSets.Count.ToString() + " PropSets:" );
            // Loop all PropertySets
            int iPropSet = 0;
            foreach( ObjectId idPropSet in idsPropSets )
            {
              ++iPropSet;
              PropertySet propSet = tr.GetObject( idPropSet, OpenMode.ForRead ) as PropertySet;
              PropertySetDefinition propSetDef = tr.GetObject( propSet.PropertySetDefinition, OpenMode.ForRead) as PropertySetDefinition;
              PropertyDefinitionCollection propDefs = propSetDef.Definitions;
              ed.WriteMessage( "\n  PropSet " + iPropSet.ToString() + "[" + propSetDef.Name + "] has " 
                + propDefs.Count.ToString() + " properties:" );
              // Loop all properties in the propSetDef  
              int iProp = 0;
              foreach( PropertyDefinition propDef in propDefs )
              {
                ++iProp;
                string value;
                if( propDef.Automatic )
                {
                  // AUTOMATIC
                  ObjectIdCollection idsDummy = new ObjectIdCollection();
                  value = propSetDef.GetValue( propDef.Id, id, idsDummy ).ToString();
                  ed.WriteMessage( "\n    " + iProp.ToString() + ".(AUTO) Name=" 
                    + propDef.Name + " Value=" + value );
                }
                else
                {
                  // STATIC
                  value = propSet.GetAt( propDef.Id ).ToString();
                  ed.WriteMessage( "\n    " + iProp.ToString() + ".(STATIC) Name=" 
                    + propDef.Name + " Value=" + value );
                }
              }
            }
          }
          // NOTE: the following contain ONLY STATIC data!
          // Dim props As PropertySetDataCollection = propSet.PropertySetData
          // Dim prop As PropertySetData
          // For Each prop In props
          // ...
          // Next prop
          tr.Commit();
        }
      }
      catch( Exception ex )
      {
        App.MsgBox( "Exception: " + ex.Message );
      }
    }
  }
}
