#Region "Header"
'      .NET Sample
'
'      Copyright (C) 2005-2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Imports"

Imports System.Windows.Forms
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.EditorInput

#End Region

' Optional to optimize the loading/searching
'   - otherwise searching the first IExtensionApplication
'<Assembly: ExtensionApplication(GetType(MyApp))> 

' Optional to optimize the loading/searching, but then only the commands within these classes are recognized
'   - otherwise searching all classes for CommandMethod
'<Assembly: CommandClass(GetType(MyCommands))> 
'<Assembly: CommandClass(GetType(MyApp))> 


' Optional Application class
Public Class App
    Implements IExtensionApplication

    Public Sub MyInitialize() Implements IExtensionApplication.Initialize

        Dim ed As Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage(vbLf & "Loading ACA.NET Intro...")

    End Sub

    Public Sub MyTerminate() Implements IExtensionApplication.Terminate
        ' Cleanup.
        ' Note: no "NETUNLOAD" available (current Microsoft limitation)
    End Sub

    ' Example of a command defined within the App class
    <CommandMethod("AppCmd")> _
    Public Sub AppCmd()
        MessageBox.Show("This is App.AppCmd().", "ACA.NET Intro")
    End Sub

End Class


' A minimal command definition
Public Class HelloAca

    <CommandMethod("HelloAca")> _
    Public Sub HelloAca()
        MessageBox.Show("Hello ACA.NET!", "ACA.NET Intro")
    End Sub

    <CommandMethod("-HelloAca")> _
    Public Sub HelloAcaCommandLine()
        Dim ed As Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Hello ACA.NET!")
    End Sub

End Class


' A class containing commands
Public Class TestCmds

    Dim count1 As Integer ' per-doc data
    Shared Count2 As Integer ' global data

    ' Constructor will be called when the first non-Shared command is called in any given Doc
    Public Sub New()
        MessageBox.Show("TestCmds Constructor!", "ACA.NET Intro")
        ' Further initialize per-doc data if needed...
    End Sub

    <CommandMethod("TestStatic")> _
    Shared Sub StaticCommand()
        MessageBox.Show("Static Command - no constructor called!", "ACA.NET Intro")
    End Sub

    <CommandMethod("TestCount")> _
    Public Sub Count()
        count1 += 1
        Count2 += 1

        'Dim msg As String
        'msg.Format("This command has been invoked: \n {0} times in this document \n {1} times in the app session", count1, Count2)
        'MessageBox.Show(msg)
        ' Also OK:
        MessageBox.Show("This command has been invoked:" + vbLf + count1.ToString + " times in this document" + _
                     vbLf + Count2.ToString + " times in the app session", "ACA.NET Intro")
    End Sub

End Class

