#Region "Header"
'      .NET Sample
'
'      Copyright (C) 2005-2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Imports"

Imports System.Reflection
'Imports System.Windows.Forms
' AutoCAD
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
'Imports AcadApp = Autodesk.AutoCAD.ApplicationServices.Application
Imports WinApp = System.Windows.Forms.Application

' To resolve TransactionManager ambiguities
Imports DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager 'wraps AcDbTransactionManager
Imports AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager 'wraps AcTransactionManager

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

#End Region

'A class containing commands
Public Class Commands1b

    ' Exception handling
    <CommandMethod("AcaNetIntroVb", "TestException", CommandFlags.Modal)> _
    Public Sub myException()

        Dim result As Double = 0

        Try
            'Convertor is in Autodesk.AutoCAD.Runtime, 
            '  so it would raise Autodesk.AutoCAD.Runtime.Exception
            result = Converter.StringToDistance("Will Fail")

        Catch e As Exception
            ' Since Autodesk.AutoCAD.Runtime.Exception is derived from System.Exception,
            '        this block fill catch *any* exception  
            'MessageBox.Show(e.GetType.ToString + ": " + e.Message() + vbLf + e.ToString, "ACA.NET Intro")
            MsgBox(e.GetType.ToString + ": " + e.Message() + vbLf + e.ToString)
            result = Double.NaN

        Finally
            ' This block is executed *always*
            MsgBox("Distance: " + result.ToString())

        End Try

    End Sub

    ' Entity selection (GetEntity) and transaction
    <CommandMethod("AcaNetIntroVb", "AcaNetSelectGeo", CommandFlags.Modal Or CommandFlags.UsePickSet)> _
    Public Sub mySelect()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim optEnt As New PromptEntityOptions(vbLf & "Select an AEC entity")
        optEnt.SetRejectMessage(vbLf & "Selected entity is NOT a AEC entity, try again...")
        optEnt.AddAllowedClass(GetType(Geo), False)  ' Geo is the base class of AEC entities. 

        Dim resEnt As PromptEntityResult = ed.GetEntity(optEnt)
        If resEnt.Status <> PromptStatus.OK Then
            ed.WriteMessage("Selection error - aborting")
            Exit Sub
        End If

        'Even to open the object for read-only we need a Transaction
        'Dim trm As DbTrMgr = AcadApp.DocumentManager.MdiActiveDocument.TransactionManager
        'Dim tr As Transaction = trm.StartTransaction()
        Dim tr As Transaction = Application.DocumentManager.MdiActiveDocument.TransactionManager.StartTransaction

        Try
            Dim obj As AcObject = tr.GetObject(resEnt.ObjectId, OpenMode.ForRead)
            MsgBox("You have selected an " & obj.GetRXClass.Name)
            tr.Commit()
        Catch
            'tr.Abort()          ' not really necessary, since Abort is called from Dispose if no Commit
        Finally
            tr.Dispose()
        End Try

    End Sub

    ' Selection set (GetSelection) and edit wall entities
    <CommandMethod("AcaNetIntroVb", "AcaNetRotateWalls", CommandFlags.Modal Or CommandFlags.UsePickSet)> _
    Public Sub mySsChange()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' Selection Options (optional)
        Dim optSS As New PromptSelectionOptions()
        optSS.MessageForAdding = "Select Walls "

        ' Selection Filter (optional)
        Dim selRB(0) As TypedValue
        selRB(0) = New TypedValue(0, "AEC_WALL")
        Dim filterSS As New SelectionFilter(selRB)

        '  Select
        Dim resSS As PromptSelectionResult = ed.GetSelection(optSS, filterSS)
        If resSS.Status <> PromptStatus.OK Then
            ed.WriteMessage("Selection error - aborting")
            Exit Sub
        End If

        Dim tr As Transaction = Application.DocumentManager.MdiActiveDocument.TransactionManager.StartTransaction

        Try
            ' Loop selected and change
            Dim selObj As SelectedObject
            For Each selObj In resSS.Value
                Dim wall As Wall = CType(tr.GetObject(selObj.ObjectId, OpenMode.ForWrite), Wall)
                ed.WriteMessage("Wall " + wall.Handle.ToString + " OLD rotation = " + wall.Rotation.ToString + vbLf)
                ' Add 9 degrees to the rotation
                wall.Rotation += System.Math.PI / 20
                ed.WriteMessage("Wall " + wall.Handle.ToString + " NEW rotation = " + wall.Rotation.ToString + vbLf)
            Next selObj

            tr.Commit()
        Catch
            tr.Abort()
        Finally
            tr.Dispose()
        End Try

    End Sub

    ' .NET Reflection to get style name and ALL properties
    <CommandMethod("AcaNetIntroVb", "AcaNetReflection", CommandFlags.Modal Or CommandFlags.UsePickSet)> _
    Public Sub myReflection()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        '  Select
        Dim resSS As PromptSelectionResult = ed.GetSelection()
        If resSS.Status <> PromptStatus.OK Then
            ed.WriteMessage("Selection error - aborting")
            Exit Sub
        End If

        Dim tr As Transaction = Application.DocumentManager.MdiActiveDocument.TransactionManager.StartTransaction
        Dim ids() As ObjectId = resSS.Value.GetObjectIds

        Try
            ' Loop selected
            Dim id As ObjectId
            For Each id In ids
                Dim ent As AcEntity = CType(tr.GetObject(id, OpenMode.ForRead), AcEntity)

                ' Entity type
                Dim msg As String = "Selected Entity is " + ent.GetType.ToString + vbCrLf

                ' Use reflection ("late binding") to check if StyleId supported
                Try
                    Dim styleId As Object = ent.GetType.InvokeMember("StyleId", Reflection.BindingFlags.GetProperty, Nothing, ent, Nothing)
                    Dim rec As DictionaryRecord = CType(tr.GetObject(CType(styleId, ObjectId), OpenMode.ForRead), DictionaryRecord)
                    msg += vbCrLf + "Style name = " + rec.Name + vbCrLf
                Catch
                    msg += vbCrLf + "NO StyleId property supported!" + vbCrLf
                End Try

                ' Get ALL properties defined on the very type
                msg += vbCrLf + ListProps(ent)

                ' Display the result
                MsgBox(msg)
            Next id

            tr.Commit()
        Catch
            tr.Abort()
        Finally
            tr.Dispose()
        End Try

    End Sub

    'helper to list properties using the reflection 
    Private Function ListProps(ByVal ent As AcEntity) As String

        Dim str As String = ""
        Dim memInfoArray() As MemberInfo = ent.GetType.FindMembers(MemberTypes.Property, _
        BindingFlags.Instance Or BindingFlags.Public, _
        Nothing, Nothing)

        Dim memInfo As MemberInfo
        For Each memInfo In memInfoArray
            Try
                Dim o As Object = ent.GetType.InvokeMember(memInfo.Name, BindingFlags.GetProperty, Nothing, ent, Nothing)
                str += "Prop." + memInfo.Name + " = " + o.ToString + vbCrLf
            Catch
                str += "Prop." + memInfo.Name + " = [exception]" + vbCrLf
            End Try
        Next memInfo

        Return str

    End Function

End Class
