#Region "Header"
'      .NET Sample
'
'      Copyright (C) 2005-2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Imports"

Imports System.Reflection
'Imports System.Windows.Forms
Imports System.Collections.Specialized
' AutoCAD
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Geometry
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.ApplicationServices
Imports Autodesk.Aec.Arch.ApplicationServices

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities with System.Windows.Forms
'Imports AcadApp = Autodesk.AutoCAD.ApplicationServices.Application

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

#End Region

'A class containing commands
Public Class Commands2a

    ' DWG Aec and Arch Settings
    <CommandMethod("AcaNetIntroVb", "AcaNetSettings", CommandFlags.Modal)> _
    Public Sub MySettings()

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tr As Transaction = db.TransactionManager.StartTransaction
        Try

            ' DrawingSetupVariables (in Autodesk.Aec.ApplicationServices)
            ' ---------------------
            '   wrap most of "Drawing Setup"
            Dim idDwgVars As ObjectId = DrawingSetupVariables.GetInstance(db, False)
            Dim dwgVars As DrawingSetupVariables = CType(tr.GetObject(idDwgVars, OpenMode.ForWrite), DrawingSetupVariables)
            Dim oldSuffix As String = dwgVars.AreaSuffix
            dwgVars.AreaSuffix = "M.sqr."
            MsgBox("Old Area suffix = " + oldSuffix + Environment.NewLine + "New Area suffix = " + dwgVars.AreaSuffix)

            ' ArchDBVariables (in Autodesk.Aec.Arch.ApplicationServices)
            ' ---------------
            '   wrap most of "Aec Object Settings" Tab in OPTIONS
            Dim idArchVars As ObjectId = ArchDBVariables.GetInstance(db, True)
            Dim o As AcObject = tr.GetObject(idArchVars, OpenMode.ForWrite)
            MsgBox("Object type (should be ArchDBVariables)= " + o.GetType.ToString)

            Dim archVars As ArchDBVariables = CType(tr.GetObject(idArchVars, OpenMode.ForWrite), ArchDBVariables)
            Dim oldR As Double = archVars.WallCleanupRadius
            archVars.WallCleanupRadius += 10
            MsgBox("Old Wall Cleanup Radius = " + oldR.ToString + vbCrLf + _
                "New Wall Cleanup Radius = " + archVars.WallCleanupRadius.ToString)

            tr.Commit()
        Catch
            tr.Abort()
        Finally
            tr.Dispose()
        End Try

    End Sub

    ' Adding new entities
    <CommandMethod("AcaNetIntroVb", "AcaNetWallAdd", CommandFlags.Modal)> _
    Public Sub WallAdd()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' Get start point - simple GetPoint signature
        Dim resPtStart As PromptPointResult = ed.GetPoint("Start point ")
        If resPtStart.Status <> PromptStatus.OK Then Exit Sub

        ' Get end point - GetPoint signature using options class to show BasePoint
        Dim optPtEnd As New PromptPointOptions(vbCrLf + "End point")
        optPtEnd.BasePoint = resPtStart.Value
        optPtEnd.UseBasePoint = True
        Dim resPtEnd As PromptPointResult = ed.GetPoint(optPtEnd)
        If resPtEnd.Status <> PromptStatus.OK Then Exit Sub

        ' Start Transaction
        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tr As Transaction = db.TransactionManager.StartTransaction
        Try
            ' Always call the two Set after New AEC entity
            Dim w As New Wall()
            w.SetDatabaseDefaults(db)       ' this WILL also call any OMF-overridden subSetDatabaseDefaults!
            w.SetToStandard(db)

            ' Note that StartPoint, etc props are read-only - must use Set!
            w.Set(resPtStart.Value, resPtEnd.Value, Vector3d.ZAxis)

            ' Add to ModelSpace
            Dim bt As BlockTable = CType(tr.GetObject(db.BlockTableId, OpenMode.ForRead), BlockTable)
            Dim btr As BlockTableRecord = _
             CType(tr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite), BlockTableRecord)
            btr.AppendEntity(w)
            ' Whenever an object added to DB, we must ALSO let the transaction know:
            tr.AddNewlyCreatedDBObject(w, True)

            tr.Commit()
        Catch
            tr.Abort()
        Finally
            tr.Dispose()
        End Try

    End Sub

    ' Styles Dictionary - eg Wall 
    <CommandMethod("AcaNetIntroVb", "AcaNetWallStyles", CommandFlags.Modal)> _
    Public Sub WallStyles()

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim tr As Transaction = db.TransactionManager.StartTransaction

        Try
            ' Get Wall Styles dictionary
            Dim dict As New DictionaryWallStyle(db)

            ' Report existing Styles
            Dim styleNames As StringCollection = dict.NamesInUse
            ed.WriteMessage(vbCrLf + "Existing wall styles:")
            Dim name As String
            For Each name In styleNames
                ed.WriteMessage(vbCrLf + "  " + name)
            Next

            ' Delete purgable styles
            Dim ids As ObjectIdCollection = dict.Records
            ' AutoCAD-check (removes any referenced)
            db.Purge(ids)
            ' Special AEC check (if dictionary "protects" some non-referenced ids, 
            '   eg see OMF AecDictDispRepConfig or can be overridden for a custom Dict)
            dict.ExtraPurgeCheck(ids)
            ed.WriteMessage(vbCrLf + vbCrLf + "There are " + ids.Count.ToString + " purgable styles to be deleted:")
            ' Loop and delete each purgable
            Dim id As ObjectId
            For Each id In ids
                Dim style As WallStyle = CType(tr.GetObject(id, OpenMode.ForWrite), WallStyle)
                ed.WriteMessage(vbCrLf + "  " + style.Name)
                style.Erase()
            Next

            tr.Commit()
        Catch
            tr.Abort()
        Finally
            tr.Dispose()
        End Try

    End Sub

    ' Anchors - check type 
    <CommandMethod("AcaNetIntroVb", "AcaNetAnchorCheck", CommandFlags.Modal Or CommandFlags.UsePickSet)> _
    Public Sub AnchorCheck()

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Do While (True)

            ' Select an ACA entity
            Dim optEnt As New PromptEntityOptions(vbLf & "Select an ACA Entity")
            optEnt.SetRejectMessage(vbLf & "Selected entity is NOT an ACA one, try again...")
            optEnt.AddAllowedClass(GetType(Geo), False)
            Dim resEnt As PromptEntityResult = ed.GetEntity(optEnt)
            If resEnt.Status <> PromptStatus.OK Then Exit Sub

            Dim tr As Transaction = db.TransactionManager.StartTransaction
            Try
                ' Open the geo (may fail due to the defect - see TCgetWall)
                Dim geo As Geo = CType(tr.GetObject(resEnt.ObjectId, OpenMode.ForRead), Geo)
                Dim msg As String = "Selected " + geo.GetType.ToString + " " + geo.Handle.ToString

                ' Check if no anchor
                If geo.AnchorId.IsNull Then
                    msg += vbCrLf + "is NOT anchored!"
                Else
                    Dim anc As Anchor = CType(tr.GetObject(geo.AnchorId, OpenMode.ForRead), Anchor)
                    msg += vbCrLf + "is anchored using " + anc.GetType.ToString + " " + anc.Handle.ToString

                    ' Check if AnchorToReference (all really are, so in pracice could've casted above)
                    ' Either of the following two will work - one uses ARX hierarchy, the other .NET
                    'If anc.GetRXClass.IsDerivedFrom(RXClass.GetClass(GetType(AnchorToReference))) Then
                    If anc.GetType.IsSubclassOf(GetType(AnchorToReference)) Then
                        Dim refAnc As AnchorToReference = CType(anc, AnchorToReference)
                        Dim refEnt As AcObject = tr.GetObject(refAnc.SingleReferenceId, OpenMode.ForRead)
                        msg += vbCrLf + "to " + refEnt.GetType.ToString + " " + refEnt.Handle.ToString
                    End If
                End If

                MsgBox(msg, "ACA.NET Intro")
                tr.Commit()
            Catch ex As Exception
                MsgBox("Exception " + ex.Message, "ACA.NET Intro")
                tr.Abort()
            Finally
                tr.Dispose()
            End Try

        Loop

    End Sub

End Class

