#Region "Header"
'      .NET Sample
'
'      Copyright (c) 2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Imports"

Imports System.Reflection
'Imports System.Windows.Forms
Imports System.Collections.Specialized
' AutoCAD
Imports Autodesk
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Geometry
' ADT (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.ApplicationServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.Arch.ApplicationServices
Imports Autodesk.Aec.PropertyData
Imports Autodesk.Aec.PropertyData.DatabaseServices

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
'Imports AcadApp = Autodesk.AutoCAD.ApplicationServices.Application

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

#End Region

'A class containing commands
Public Class Commands4

	' Check which display properties a Wall uses for all current dispreps and list some display components
    ' Display System is exposed in ACA 2008. In previous version, this is done through COM Interop API. 
    ' 
    <CommandMethod("AcaNetIntro", "AcaDispProps", CommandFlags.Modal)> _
 Public Sub DisplayOverrides()

        ' Minimum error checking!

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
        Dim trans As Transaction = db.TransactionManager.StartTransaction()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' select a wall and get the wall style Id
        Dim optEnt As New PromptEntityOptions(vbLf & "Select a Wall")
        optEnt.SetRejectMessage(vbLf & "Selected entity is NOT a Wall, try again...")
        optEnt.AddAllowedClass(GetType(Wall), True)

        Dim resEnt As PromptEntityResult = ed.GetEntity(optEnt)
        If resEnt.Status <> PromptStatus.OK Then
            ed.WriteMessage("Selection error - aborting")
            Exit Sub
        End If

        ' get the wall object. 
        Dim wall As Wall = trans.GetObject(resEnt.ObjectId, OpenMode.ForRead)
        ed.WriteMessage("You have selected an " & wall.GetRXClass.Name & vbCrLf)

        ' get its style
        Dim wallstyle As Autodesk.Aec.Arch.DatabaseServices.WallStyle = trans.GetObject(wall.StyleId, OpenMode.ForRead)

        MsgBox("Got the Wall and WallStyle objects: handle=" & wall.Handle.ToString & ", style=" & wallstyle.Name)

        '  first get the display rep manager - center of the display system. 
        Dim dispRepMgr As DisplayRepresentationManager = New DisplayRepresentationManager(db)

        '  get all DispReps for wall used in the current viewport and loop them
        Dim ids As ObjectIdCollection = dispRepMgr.GetDisplayRepresentationIdsFromCurrentViewport(wall.GetRXClass())
        ' ed.WriteMessage("wall disp rep count = " & ids.Count & vbCrLf)


        Dim dispRep As DisplayRepresentation
        Dim s As String = ""

        Try
            For Each id As ObjectId In ids
                Dim dps As DisplayProperties = Nothing
                dispRep = trans.GetObject(id, OpenMode.ForRead)

                s = "--  Display Properties Override  --" & vbCrLf & vbCrLf
                s += "Display Representation Name: " & dispRep.Name & ", Display Name: " & dispRep.DisplayName & vbCrLf


                Dim rcProps As RXClass = dispRep.DisplayPropertiesClass
                'ed.WriteMessage("Display Props Class: " & rcProps.Name & vbCrLf)
                s += "Display Props Class: " & rcProps.Name & vbCrLf

                ' find override
                ' (1) first check at element level.
                Dim overId As ObjectId = dispRep.FindDisplayPropertiesOverride(wall)
                If (overId <> ObjectId.Null) Then
                    dps = trans.GetObject(overId, OpenMode.ForRead)
                    s += "ELEMENT OVERRIDE: "
                Else
                    Dim overstyleId As ObjectId = dispRep.FindDisplayPropertiesOverride(wallstyle)
                    If (overstyleId <> ObjectId.Null) Then
                        dps = trans.GetObject(overstyleId, OpenMode.ForRead)
                        s += "STYLE OVERRIDE: "
                    Else
                        ' (3) if not at element or style, then must be at drawing default. 
                        Dim defaultDpsId As ObjectId = dispRep.DefaultDisplayPropertiesId
                        If Not defaultDpsId.IsNull Then
                            dps = trans.GetObject(defaultDpsId, OpenMode.ForRead)
                            s += "DRAWING DEFAULT: "
                        End If
                    End If
                End If
                s += dps.Name & " Display Name:" & dps.DisplayName & vbCrLf

                MsgBox(s) ' report result
                s = "--  Display Components --" & vbCrLf & vbCrLf

                '  show a few display component

                Dim sc As System.Collections.Specialized.StringCollection = New System.Collections.Specialized.StringCollection()
                Dim components() As DisplayComponent = dps.GetDisplayComponents(sc)
                If components.Length = 0 Then
                    Continue For
                End If

                Dim i As Integer
                For i = 0 To components.Length - 1

                    Dim dispComp As DisplayComponent = components(i)
                    Dim name As String = sc(i)

                    ' Hatch or entity component? 
                    If (TypeOf (dispComp) Is DisplayComponentHatch) Then
                        Dim compHatch As DisplayComponentHatch = dispComp
                        ' list a few properties 
                        s += "HATCH Component: " & name
                        s += ": Visibility=" & compHatch.IsVisible.ToString
                        s += ", Color=" & compHatch.ColorIndex.ToString
                        s += ", Pattern=" & compHatch.PatternName & vbCrLf

                    ElseIf (TypeOf (dispComp) Is DisplayComponentEntity) Then
                        Dim compEnt As DisplayComponentEntity = dispComp
                        ' list a few properties 
                        s += "ENTITY Component: " & name
                        s += ": Visibility=" & compEnt.IsVisible.ToString
                        s += ", Color=" & compEnt.ColorIndex.ToString & vbCrLf

                    End If

                Next i

                MsgBox(s) ' report result 

            Next id

        Catch
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

    End Sub

End Class
