//
//  Copyright 2004-2008 by Autodesk, Inc.
//
//  Permission to use, copy, modify, and distribute this software in
//  object code form for any purpose and without fee is hereby granted, 
//  provided that the above copyright notice appears in all copies and 
//  that both that copyright notice and the limited warranty and
//  restricted rights notice below appear in all supporting 
//  documentation.
//
//  AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
//  AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
//  DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
//  UNINTERRUPTED OR ERROR FREE.
//
//  Use, duplication, or disclosure by the U.S. Government is subject to 
//  restrictions set forth in FAR 52.227-19 (Commercial Computer
//  Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
//  (Rights in Technical Data and Computer Software), as applicable.
// 
//  Last updated: Aug 31, 2008 by MH. Migrated to use .NET API in ACA 2009. (was COM Interop) 
// 

using System;                           // Double
using System.Windows.Forms;             // MessageBox

using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Recipe;
using Autodesk.Aec.Recipe.Utilities;
using Autodesk.Aec.Arch.DatabaseServices;

using AcDb = Autodesk.AutoCAD.DatabaseServices; 

using TransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager;
using ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId;
using DtlUtils = Autodesk.Aec.Recipe.Utilities;


// NOTE: do not use a namespace
// 

//  C# version of recipe to draw a "Rectangle" 
// 

public class Class1
{
    public static void Test1(object acadApp)
    {
        MessageBox.Show("Greetings from the C# Test!");
    }

    public static void DrawRectangle(object acadObj)
    {
        // debug
        //Test1(acadObj); 
    
        RecipeInformationAccess rcpInfo = new RecipeInformationAccess();

		// (1) Get the raw dimensions from the database (.mdb) 
        // 
        double x = DtlUtils.Database.GetFieldValueDoubleUnitDependent("SIZE_X");
        double y = DtlUtils.Database.GetFieldValueDoubleUnitDependent("SIZE_Y");

        // (2) Get user input from the palette
        // 
        String strJig = DtlUtils.Palette.GetPropertyValueString("jig");
        bool bHatch = DtlUtils.Palette.GetPropertyValueBool("hatch");

        // (3) Get some parameters from the recipe spec
        // MH:  Recipe here means Recipe Spec (.xml) file.  
        // 
        String strPosition = DtlUtils.Recipe.GetDataValueString("label", "Position");
        double dHeightFactor = DtlUtils.Recipe.GetDataValueDouble("label", "HeightFactor");

        // (4) build a component, using compBuilder. It will take care of many house keeping tasks.  
        // 
        ComponentBuilderBlockDefinition compBuilder = new ComponentBuilderBlockDefinition(rcpInfo);
        compBuilder.Start();

        try
        {
            MakeRectangle(rcpInfo, compBuilder, x, y, bHatch, dHeightFactor, strPosition); 
        }
        catch
        {
            compBuilder.Abort();
        }
        finally
        {
            compBuilder.EndOrAbort();
        }

        // (5) start the jig interaction. 
        // 

        if (strJig == "jigStamp")   
        {
            Recipe.ReturnStatus = JigUtilities.StampIt(compBuilder);
        }
        else if (strJig == "jigArray")
        {
            Recipe.ReturnStatus = JigUtilities.LinearArrayIt(compBuilder);
        }
        else if (strJig == "jigBookends")
        {
            // we need a bit more work for bookends jig, creating additional components at the both ends. 
            // 
            ComponentBuilderBlockDefinition compBuilderStart = new ComponentBuilderBlockDefinition(rcpInfo);
            compBuilderStart.Start();
            ComponentBuilderBlockDefinition compBuilderEnd = new ComponentBuilderBlockDefinition(rcpInfo);
            compBuilderEnd.Start();

            try
            {
                MakeRectangleBookends(rcpInfo, compBuilderStart, compBuilderEnd, x, y, bHatch);
            }
            catch
            {
                compBuilderStart.Abort();
                compBuilderEnd.Abort();
            }
            finally
            {
                compBuilderStart.EndOrAbort();
                compBuilderEnd.EndOrAbort();
            }

            // create three block refs: at the both end and in between. 
            AcDb.BlockReference middleBlock = compBuilder.GetNewReference();
            AcDb.BlockReference startBlock = compBuilderStart.GetNewReference();
            AcDb.BlockReference endBlock = compBuilderEnd.GetNewReference();

            // create another compBuilder to use with bookends jig. 
            ComponentBuilderCurrentSpace builder = new ComponentBuilderCurrentSpace(rcpInfo);

            // now insert the blocks that we made. 
            Recipe.ReturnStatus = JigUtilities.Bookends(builder, startBlock, endBlock, middleBlock, false);
        }
        
    }

    //  define a middle rectangle 
    //
    public static void MakeRectangle(RecipeInformationAccess rcpInfo, ComponentBuilder compBuilder, Double x, Double y, bool bHatch, double dHeightFactor, String strPosition)
    {
        // Create the polyline
        // 
        Point2dCollection pts = new Point2dCollection();
        pts.Add(new Point2d(0.0, 0.0) ); 
        pts.Add(new Point2d(x, 0.0) ); 
        pts.Add(new Point2d(x, y) ); 
        pts.Add(new Point2d(0.0, y) ); 

        Polyline poly = new Polyline();
        compBuilder.SetToDefaultProperties(poly);
        compBuilder.AddPolyline(poly, pts, true);

        // Hatch, if needed 
        // 
        if (bHatch)
        {
            Hatch hat = new Hatch();
            hat.SetHatchPattern(HatchPatternType.PreDefined, "ANSI31");
            compBuilder.AddHatchToPreviousBoundary(hat, true);  
        }

        // Add text
        // 
        double[] pnt = new double[3]; pnt[0] = x / 2; pnt[2] = 0;

        AttachmentPoint align;
        if (strPosition == "Top")
        {
            pnt[1] = y;
            align = AttachmentPoint.BottomCenter;  
        }
        else if (strPosition == "Bottom")
        {
            pnt[1] = -y * dHeightFactor / 5;
            align = AttachmentPoint.TopCenter;  
        }
        else
        {
            pnt[1] = y / 2;
            align = AttachmentPoint.MiddleCenter;  
        }

        string strText = x.ToString() + "x" + y.ToString() ;

        MText txt = new MText(); 
        txt.Contents = strText;
        txt.Location = new Point3d(pnt);
        txt.TextHeight = y * dHeightFactor;
        txt.Attachment = align;
        compBuilder.AddToDatabase(txt); 

    }

    //  defines rectangles at both ends for bookends jig.  
    //
    public static void MakeRectangleBookends(RecipeInformationAccess rcpInfo, ComponentBuilder compBuilderStart, ComponentBuilder compBuilderEnd, Double x, Double y, bool bHatch)
    {
        // (0) Define the bookend geometry 
        //
        //          cutline1
        //       _____ /_____
        //      |     /      |
        // left |    /__     | start (closed plines)
        //      |       /    |
        //      |_____ /_____|
        //            /
        //          cutline2
        //      (just the tips)
        //

        Point2dCollection right = new Point2dCollection();
        right.Add(new Point2d(x / 2.0, 0.0));
        right.Add(new Point2d(x, 0.0));
        right.Add(new Point2d(x, y));
        right.Add(new Point2d(x / 2.0, y));
        right.Add(new Point2d(x / 4.0, y / 2.0));
        right.Add(new Point2d(3.0 * x / 4.0, y / 2.0));
        right.Add(new Point2d(x / 2.0, 0.0)); 

        Point2dCollection left = new Point2dCollection();
        left.Add(new Point2d(x / 2.0, 0.0));
        left.Add(new Point2d(0, 0.0));
        left.Add(new Point2d(0, y));
        left.Add(new Point2d(x / 2.0, y));
        left.Add(new Point2d(x / 4.0, y / 2.0));
        left.Add(new Point2d(3.0 * x / 4.0, y / 2.0));
        left.Add(new Point2d(x / 2.0, 0.0)); 

        Point2dCollection cutline1 = new Point2dCollection();
        cutline1.Add(new Point2d(x / 4.0, -y / 2.0));
        cutline1.Add(new Point2d(3 * x / 4.0, y / 2.0));

        Point2dCollection cutline2 = new Point2dCollection();
        cutline2.Add(new Point2d(x / 4.0,  y / 2.0));
        cutline2.Add(new Point2d(3 * x / 4.0, 3 * y / 2.0));


        // (1) Create the first bookend
        //         /_____
        //        /      |
        //       /__     |
        //          /    |
        //         /_____|
        //        /

        Polyline polyCutline1 = new Polyline();
        compBuilderStart.SetToDefaultProperties(polyCutline1);
        compBuilderStart.AddPolyline(polyCutline1, cutline1, true);

        Polyline polyCutline2 = new Polyline();
        compBuilderStart.SetToDefaultProperties(polyCutline2);
        compBuilderStart.AddPolyline(polyCutline2, cutline2, true);

        Polyline polyRight = new Polyline();
        compBuilderStart.SetToDefaultProperties(polyRight);
        compBuilderStart.AddPolyline(polyRight, right, true);

        // Hatch, if needed 
        // 
        if (bHatch)
        {
            Hatch hatStart = new Hatch();
            hatStart.SetHatchPattern(HatchPatternType.PreDefined, "ANSI31");
            compBuilderStart.AddHatchToPreviousBoundary(hatStart, true);
        }

        // (2) Create the second bookend
        //   _____ /
        //  |     /
        //  |    /__
        //  |       /
        //  |_____ /
        //        /

        Polyline polyCutline11 = new Polyline();
        compBuilderEnd.SetToDefaultProperties(polyCutline11);
        compBuilderEnd.AddPolyline(polyCutline11, cutline1, true);

        Polyline polyCutline22 = new Polyline();
        compBuilderEnd.SetToDefaultProperties(polyCutline22);
        compBuilderEnd.AddPolyline(polyCutline22, cutline2, true);

        Polyline polyLeft = new Polyline();
        compBuilderEnd.SetToDefaultProperties(polyLeft);
        compBuilderEnd.AddPolyline(polyLeft, left, true);

        if (bHatch)
        {
            Hatch hatEnd = new Hatch();
            hatEnd.SetHatchPattern(HatchPatternType.PreDefined, "ANSI31");
            compBuilderEnd.AddHatchToPreviousBoundary(hatEnd, true);
        }

    }

}