// This is the main DLL file.
//
//  Copyright 2004-2008 by Autodesk, Inc.
//
//  Permission to use, copy, modify, and distribute this software in
//  object code form for any purpose and without fee is hereby granted, 
//  provided that the above copyright notice appears in all copies and 
//  that both that copyright notice and the limited warranty and
//  restricted rights notice below appear in all supporting 
//  documentation.
//
//  AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
//  AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
//  DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
//  UNINTERRUPTED OR ERROR FREE.
//
//  Use, duplication, or disclosure by the U.S. Government is subject to 
//  restrictions set forth in FAR 52.227-19 (Commercial Computer
//  Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
//  (Rights in Technical Data and Computer Software), as applicable.
// 
// 

// 
//  Manged C++ version of Rectablge Recipe.
// 
//  Note: This has not been migrated to .NET API in ACA 2009. Still in COM Interop. 
//  Managed C++ is less polular as .NET language. low demand. Not sure if we want to migrate it. 
//  This also uses an old syntax. Want to migrate to clr if we do.   
//  Please refere to C# example. That's probably the closet you can get.  (2008/09/01 by MH)     
// 

#include "stdafx.h"

#include "TestDtl_MCPP.h"

void Class1::Test(Object* acadObj)
{
    AcadApplication* acadApp = dynamic_cast<AcadApplication*>(acadObj);

    String* strCaption = acadApp->get_Caption();
    MessageBox::Show(strCaption);
}

void Class1::DrawRectangle(Object* acadObj)
{
	// debug. MH.  
	//MessageBox::Show("Hello from MCPP");

    AcadApplication * acadApp;
    acadApp = dynamic_cast<AcadApplication*>(acadObj);
    
    // Get the details application object
    AecDtlApplication* dtlApp;
    dtlApp = dynamic_cast<AecDtlApplication*>(acadApp->GetInterfaceObject("AecX.AecDtlApplication.5.7"));
    
    // Get the raw dimensions from the database
    double x = 0;
    double y = 0;
    x = dtlApp->Database->GetFieldValueDoubleUnitDep("SIZE_X");
    y = dtlApp->Database->GetFieldValueDoubleUnitDep("SIZE_Y");
    
    // Get user input from the palette
    String* strJig;
    bool bHatch;
    strJig = dtlApp->Palette->GetPropertyValueStr("jig");
    bHatch = dtlApp->Palette->GetPropertyValueBool("hatch");

    // Get some parameters from the recipe spec
    String* strPosition;
    double dHeightFactor;
    strPosition = dtlApp->Recipe->GetDataValueStr("label", "Position");
    dHeightFactor = dtlApp->Recipe->GetDataValueDouble("label", "HeightFactor");

    // Create an empty block def that will hold our rectangle
    AcadBlock* blkDef = CreateEmptyBlockDef(acadApp);

    // Create the polyline
    Double points[] = new Double[8];
    points[0] = 0; points[1] = 0;
    points[2] = x; points[3] = 0;
    points[4] = x; points[5] = y;
    points[6] = 0; points[7] = y;

    AcadLWPolyline* lwpline;
    lwpline = blkDef->AddLightWeightPolyline(points);
    lwpline->Closed = true;

    // Hatch, if necessary
    if (bHatch) {
        AcadHatch* hatch = blkDef->AddHatch(AcPatternType::acHatchPatternTypePreDefined, "ANSI31", true, __box(AcHatchObjectType::acHatchObject));
        AcadLWPolyline* varObjs[] = new AcadLWPolylineClass*[1];
        varObjs[0] = lwpline;
        hatch->AppendOuterLoop(varObjs);
        hatch->Evaluate();
        hatch->Update();
    }

    // Add text
    Double pnt[] = new Double[3]; pnt[0] = x / 2; pnt[2] = 0;
    AcAlignment align;
    if (strPosition->Equals("Top")) {
        pnt[1] = y;
        align = AcAlignment::acAlignmentBottomCenter;
    }
    else if (strPosition->Equals("Bottom")) {
        pnt[1] = -y * dHeightFactor / 5;
        align = AcAlignment::acAlignmentTopCenter;
    }
    else {
        pnt[1] = y / 2;
        align = AcAlignment::acAlignmentMiddleCenter;
    }
        
    String* strText = String::Concat(x.ToString(), "x", y.ToString());
    AcadText* text = blkDef->AddText(strText, pnt, y * dHeightFactor);
    text->Alignment = align;
    text->TextAlignmentPoint = pnt;
    text->Rotation = 3.14159 / 2;

    // Pass off to the jig
    AecDtlPromptStatus retStat;
    if (strJig->Equals("jigArray"))
        retStat = dtlApp->Jig->LinearArrayBlock(blkDef);
    else if (strJig->Equals("jigBookends"))
    {
        AcadBlock* startBlkDef = CreateEmptyBlockDef(acadApp);
        AcadBlock* endBlkDef = CreateEmptyBlockDef(acadApp);
        MakeRectangleBookends(x, y, startBlkDef, endBlkDef, bHatch);
        retStat = dtlApp->Jig->BookendsBlock(startBlkDef, endBlkDef, blkDef);
    }
    else
        retStat = dtlApp->Jig->StampBlock(blkDef);
    
    dtlApp->Recipe->ReturnStatus = retStat;
}

void Class1::MakeRectangleBookends(Double x, Double y, AcadBlock* startBlkDef, AcadBlock* endBlkDef, bool bHatch)
{
    // Define the bookend geometry
    //
    //          cutline1
    //       _____ /_____
    //      |     /      |
    // left |    /__     | start (closed plines)
    //      |       /    |
    //      |_____ /_____|
    //            /
    //          cutline2
    //      (just the tips)
    //
    
    // Define the bookend geometry
    Double right[] = new Double[14];
    right[0] = x / 2 ; right[1] = 0;
    right[2] = x ; right[3] = 0;
    right[4] = x ; right[5] = y;
    right[6] = x / 2 ; right[7] = y;
    right[8] = x / 4 ; right[9] = y / 2;
    right[10] = 3 * x / 4 ; right[11] = y / 2;
    right[12] = x / 2 ; right[13] = 0;

    Double left[] = new Double[14];
    left[0] = x / 2 ; left[1] = 0;
    left[2] = 0 ; left[3] = 0;
    left[4] = 0 ; left[5] = y;
    left[6] = x / 2 ; left[7] = y;
    left[8] = x / 4 ; left[9] = y / 2;
    left[10] = 3 * x / 4 ; left[11] = y / 2;
    left[12] = x / 2 ; left[13] = 0;

    Double cutline1[] = new Double[4];
    cutline1[0] = x / 4 ; cutline1[1] = -y / 2;
    cutline1[2] = 3 * x / 4 ; cutline1[3] = y / 2;

    Double cutline2[] = new Double[4];
    cutline2[0] = x / 4 ; cutline2[1] = y / 2;
    cutline2[2] = 3 * x / 4 ; cutline2[3] = 3 * y / 2;

    // Create the first bookend
    //         /_____
    //        /      |
    //       /__     |
    //          /    |
    //         /_____|
    //        /
    AcadLWPolyline* lwpline;
    lwpline = startBlkDef->AddLightWeightPolyline(cutline1);
    lwpline = startBlkDef->AddLightWeightPolyline(cutline2);
    lwpline = startBlkDef->AddLightWeightPolyline(right);
    if (bHatch) 
    {
        AcadHatch* hatch = startBlkDef->AddHatch(AcPatternType::acHatchPatternTypePreDefined, "ANSI31", true, __box(AcHatchObjectType::acHatchObject));
        AcadLWPolyline* varObjs[] = new AcadLWPolylineClass*[1];
        varObjs[0] = lwpline;
        hatch->AppendOuterLoop(varObjs);
        hatch->Evaluate();
        hatch->Update();
    }

    // Create the second bookend
    //   _____ /
    //  |     /
    //  |    /__
    //  |       /
    //  |_____ /
    //        /
    lwpline = endBlkDef->AddLightWeightPolyline(cutline1);
    lwpline = endBlkDef->AddLightWeightPolyline(cutline2);
    lwpline = endBlkDef->AddLightWeightPolyline(left);
    if (bHatch) 
    {
        AcadHatch* hatch = endBlkDef->AddHatch(AcPatternType::acHatchPatternTypePreDefined, "ANSI31", true, __box(AcHatchObjectType::acHatchObject));
        AcadLWPolyline* varObjs[] = new AcadLWPolylineClass*[1];
        varObjs[0] = lwpline;
        hatch->AppendOuterLoop(varObjs);
        hatch->Evaluate();
        hatch->Update();
    }

}

// ** Create an empty block def that we can use to fill up by another function
// ** Right now it just uses an anonymous block name, but we'll have to have it
// ** do whatever the main routines do when we get the Anon block issue solved.

AcadBlock* Class1::CreateEmptyBlockDef(AcadApplication* acadApp)
{
    AcadDocument* acadDoc = acadApp->ActiveDocument;
    String* blkName = "*U";

    // Define the block
    Double basePt[] = new Double [3];
    basePt[0] = 0.0; 
    basePt[1] = 0.0; 
    basePt[2] = 0.0;
    
    // Add the block to the blocks collection
    return acadDoc->Blocks->Add(basePt, blkName);
}
