// $Header: //SandBox/awej/Content/AllUsers/RcpSamples/Source/SampRcp_CPP/Inc/SampEdJigEntCircleJoist.h#1 $
// $Author: awej $ $DateTime: 2004/05/05 14:47:54 $ $Change: 11180 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

#ifndef AEC_SAMPEDJIGENTCIRCLEJOIST_H
#define AEC_SAMPEDJIGENTCIRCLEJOIST_H

#include "AecRcpBase/AecEdJigEntLineExtend.h"

/****************************************************************************
**
**  CLASS SampEdJigEntCircleJoist
**      A JigEnt is a "fake" entity used to draw dynamic feedback while a Jig
**  is active.  It is only needed for complex components that want to provide
**  rich, interactive feedback.
**
**  You should structure JigEnts so that they do the calculations necessary for
**  the final entities to be created in the database.  Override the ::calculate()
**  function to figure out what needs to be drawn.  Override the ::drawInEcs()
**  function to give visual feedback while the Jig is active.  The Jigs are ECS based
**  so that you can always draw with a simple, fixed coordinate system.  Any 
**  geometry you generate will be transformed to the correct location by the
**  worldDraw() function or the CompBldr using the Jig's established ECS transform.
**
**
*************************************/

class SampEdJigEntCircleJoist : public AecEdJigEntLineExtend {

public:
	enum PointType {
		kTopPline   = 0,
		kBottomPline,
		kBreakLine
	};
                                SampEdJigEntCircleJoist(double depth, double chordWidth, double webDia);
    virtual                     ~SampEdJigEntCircleJoist();

	virtual void				calculate();
	virtual void				getEcsOutline(AcGePoint2dArray& pts, PointType ptype);

        // we'll return these directly so we can create them efficiently
    AcDbVoidPtrArray&           curvesVisible() { return m_dbCurvesVisible; }
    AcDbVoidPtrArray&           curvesHidden()  { return m_dbCurvesHidden; }

protected:
        // data members
    double                      m_depth;
	double                      m_chordDepth;
	double                      m_webDia;
	AcGePoint3d					m_breakLine[6];
    AcGePoint3d                 m_breakLineOrigin[6];
	AcGePoint3d					m_topPline[4];
	AcGePoint3d					m_bottomPline[4];
	AcDbVoidPtrArray			m_dbCurvesVisible;
	AcDbVoidPtrArray			m_dbCurvesHidden;
	double						m_breakLineFactor;

	virtual void                drawInEcs(AcGiWorldDraw* pWorldDraw);

	void						setupBreakPoints();

	void						calcInterior(const AcDbPolyline* breakPline);
    Acad::ErrorStatus           trimCircle(AcDbCurve* circleToTrim,
                                        const AcDbLine* trimLineTop, const AcDbLine* trimLineBottom,
                                        const AcDbCurve* breakLine);
    AcGePoint3d                 midpoint(const AcDbCurve* curve);
    void                        freeDbCurves();
};

#endif // AEC_SAMPEDJIGENTCIRCLEJOIST_H
