// $Header: //SandBox/awej/Content/AllUsers/RcpSamples/Source/SampRcp_CPP/Src/SampDtlCompApp.cpp#1 $
// $Author: awej $ $DateTime: 2004/05/05 14:47:54 $ $Change: 11180 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

#include "stdafx.h"
#include "SampDtlCompApp.h"
#include "SampRcpMisc.h"
#include "SampRcpCoredBrick.h"
#include "SampRcpBulgingMortar.h"
#include "SampRcpCobblestone.h"
#include "SampRcpCircleJoists.h"

#include "AecLogicalAppNames.h"
#include "AecRcpBase/AecRcpAppsTable.h"
#include "AecRcpBase/AecRcpFuncTable.h"
#include "AecUiBase/AecUiBaseServices.h"

static AFX_EXTENSION_MODULE AecTestRcpDll;
HINSTANCE _hdllInstance = NULL;
SampDtlCompApp* ThisApp = NULL;


extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
        // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH) {
        _hdllInstance =  hInstance;

        if (!AfxInitExtensionModule(AecTestRcpDll, hInstance))
            return 0;

        ThisApp = new SampDtlCompApp;

            // Insert this DLL into the resource chain
        new CDynLinkLibrary(AecTestRcpDll);
    }
    else if (dwReason == DLL_PROCESS_DETACH) {
        AfxTermExtensionModule(AecTestRcpDll);
        if (ThisApp) {
            delete ThisApp;
            ThisApp = NULL;
            _hdllInstance = NULL;
        }
    }
    
    return 1;   // ok
}


/////////////////////////////////////////////////////////////////////////////


/****************************************************************************
**
**  acrxEntryPoint:
**
**
*************************************/

extern "C" AcRx::AppRetCode
acrxEntryPoint(AcRx::AppMsgCode msg, void* pPkt)
{
    return ThisApp->entryPoint(msg, pPkt);
}

/****************************************************************************
**
**  SampDtlCompApp::getApp
**      static function to get access to this module in general
**
**
*************************************/

SampDtlCompApp*
SampDtlCompApp::getApp()
{
    return ThisApp;
}

////////////////////////////////////////////////////////////////////////////
/////////////// End of Standard Template For ArxApp ////////////////////////
////////////////////////////////////////////////////////////////////////////

AEC_ACRX_NO_CONS_DEFINE_MEMBERS(SampDtlCompApp, AecAppArx, LAN_SAMPRCPCPP, -1, -1);

/****************************************************************************
**
**  SampDtlCompApp::SampDtlCompApp
**
**
*************************************/

SampDtlCompApp::SampDtlCompApp()
:   AecAppArx(LAN_SAMPRCPCPP_STR, _hdllInstance, AecTestRcpDll)
{
}

/****************************************************************************
**
**  SampDtlCompApp::SampDtlCompApp
**
**
*************************************/

Acad::ErrorStatus 
SampDtlCompApp::registerServices()
{
    Acad::ErrorStatus es = Acad::eOk;
    es = AecUiBaseServices::registerService(this);

    es = AecRcpBaseServices::registerService(this);
    if (es == Acad::eOk)
        registerRcpFuncObjects();

    return es;
}

/****************************************************************************
**
**  SampDtlCompApp::unRegisterServices
**
**
*************************************/

void
SampDtlCompApp::unRegisterServices()
{
    unRegisterRcpFuncObjects();

    AecRcpBaseServices::unRegisterService(this);
    AecUiBaseServices::unRegisterService(this);
}

/****************************************************************************
**
**  SampDtlCompApp::registerRcpFuncObjects
**      Here we need to register our RcpFuncs.  These are the functions referenced
**  from the XML RecipeSpec file with the following construct:
**
**		<Implementation>
**			<ObjectARX>
**				<AppName>SampRcpCpp40</AppName>
**				<RcpFunc>CircleJoistSection</RcpFunc>
**			</ObjectARX>
**		</Implementation>
**
**  When the framework goes to look up the correct RcpFunc, it will find the AecRcpFuncTable
**  with logical appname "SampRcpCpp40", and then find the function registered for "CircleJoistSection".
**
**  This is the function that fills up our table of RcpFuncs for this app.
**
*************************************/

void
SampDtlCompApp::registerRcpFuncObjects()
{
        // get the global RcpApps table
    AecRcpAppsTable* appsTbl = AecRcpBaseServices::service()->rcpAppsTable();

        // make a new entry for our app, add our functions to it
        // and then add our funcTable to the appsTable
    AecRcpFuncTable* funcTbl = new AecRcpFuncTable(appName());
    funcTbl->addRcpFuncClass(new SampRcpMisc);
    funcTbl->addRcpFuncClass(new SampRcpCoredBrick);
    funcTbl->addRcpFuncClass(new SampRcpBulgingMortar);
    funcTbl->addRcpFuncClass(new SampRcpCobblestone);
    funcTbl->addRcpFuncClass(new SampRcpCircleJoist);

    appsTbl->addAppFuncTable(funcTbl);
}

/****************************************************************************
**
**  SampDtlCompApp::unRegisterRcpFuncObjects
**      When we unload our app, we need to unregister our functions.  Since they
**  are all neatly contained within our AecRcpFunTable, we simply need to remove
**  it from the system.
**
**
*************************************/

void
SampDtlCompApp::unRegisterRcpFuncObjects()
{
        // simply remove our table from the list
    AecRcpAppsTable* appsTbl = AecRcpBaseServices::service()->rcpAppsTable();
    appsTbl->removeAppFuncTable(appName());
}

/****************************************************************************
**
**  SampDtlCompApp::getRequiredModules
**
**
*************************************/

void
SampDtlCompApp::getRequiredModules(AecRmCStringArray& reqApps)
{
    reqApps.Add(LAN_AECRCPBASE_STR);
}

/****************************************************************************
**
**  SampDtlCompApp::initApp
**
**
*************************************/

AcRx::AppRetCode
SampDtlCompApp::initApp()
{
    acrxDynamicLinker->registerAppMDIAware(acadAppPtr());
    return AcRx::kRetOK;
}


