// $Header: //SandBox/awej/Content/AllUsers/RcpSamples/Source/SampRcp_CPP/Src/SampEdJigEntCircleJoist.cpp#1 $
// $Author: awej $ $DateTime: 2004/05/05 14:47:54 $ $Change: 11180 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

#include "stdafx.h"
#include "SampEdJigEntCircleJoist.h"

/****************************************************************************
**
**  SampEdJigEntCircleJoist::SampEdJigEntCircleJoist
**
**
*************************************/

SampEdJigEntCircleJoist::SampEdJigEntCircleJoist(double depth, double chordDepth, double webDia)
:   m_depth(depth),
    m_chordDepth(chordDepth),
    m_webDia(webDia),
    m_dbCurvesVisible(56),
    m_dbCurvesHidden(56)
{
    AEC_ASSERT(m_depth != 0.0);
    AEC_ASSERT(m_chordDepth != 0.0);
    AEC_ASSERT(m_webDia != 0.0);

    setupBreakPoints();
}

/****************************************************************************
**
**  SampEdJigEntCircleJoist::~SampEdJigEntCircleJoist
**
**
*************************************/

SampEdJigEntCircleJoist::~SampEdJigEntCircleJoist()
{
    freeDbCurves();
}

/****************************************************************************
**
**  SampEdJigEntCircleJoist::freeDbCurves
**
**
*************************************/

void
SampEdJigEntCircleJoist::freeDbCurves()
{
    int len = m_dbCurvesVisible.length();
    for (int i=0; i<len; i++) {
        delete static_cast<AcDbCurve*>(m_dbCurvesVisible[i]);
    }

    m_dbCurvesVisible.setLogicalLength(0);

    len = m_dbCurvesHidden.length();
    for (int i=0; i<len; i++) {
        delete static_cast<AcDbCurve*>(m_dbCurvesHidden[i]);
    }

    m_dbCurvesHidden.setLogicalLength(0);
}

/****************************************************************************
**
**  SampEdJigEntCircleJoist::drawInEcs
**
**
*************************************/

void
SampEdJigEntCircleJoist::drawInEcs(AcGiWorldDraw* pWorldDraw)
{
	// purposely do not call base class (we'll draw our own line)

	pWorldDraw->geometry().polyline(4, m_topPline);
	pWorldDraw->geometry().polyline(6, m_breakLine);
	pWorldDraw->geometry().polyline(4, m_bottomPline);

        // draw the arcs for the circle joists
    AcDbDatabase* db = acdbCurDwg();
    AcDbEntity* tmpEnt;
    int len = m_dbCurvesVisible.length();
    for (int i=0; i<len; i++) {
        tmpEnt = static_cast<AcDbEntity*>(m_dbCurvesVisible[i]);
        tmpEnt->setDatabaseDefaults(db);
        pWorldDraw->geometry().draw(tmpEnt);
    }

    drawDirectionAid(pWorldDraw, m_depth, m_depth * 0.25);
    if (m_promptSeq == 1)
        drawExtensionAid(pWorldDraw, m_depth, true);                   
}

/****************************************************************************
**
**  SampEdJigEntCircleJoist::getEcsOutline
**  
**
*************************************/

void
SampEdJigEntCircleJoist::getEcsOutline(AcGePoint2dArray& pts, PointType ptype)
{
	pts.setLogicalLength(0);
	int count;
	if (ptype == kTopPline || ptype == kBottomPline)
		count = 4;
	else if(ptype == kBreakLine)
		count = 6;
	else {
		AEC_ASSERT(0);
	}

	AcGePoint3d* pData;

	switch(ptype) {
	    case kTopPline: 
		    pData = m_topPline;
		    break;
	    case kBottomPline:
		    pData = m_bottomPline;
		    break;
	    case kBreakLine:
		    pData = m_breakLine;
		    break;
	    default:
		    AEC_ASSERT(0);
	}

	for (int i=0; i<count; i++){
		pts.append(Aec::point3dTo2d(pData[i]));
	}
}

/****************************************************************************
**
**  SampEdJigEntCircleJoist::calculate
**      figure out what the outline of the boundary is (in ECS)
**
**
*************************************/

void
SampEdJigEntCircleJoist::calculate()
{
	AecEdJigEntLineExtend::calculate();   // let base class do its thing

	AcGePoint3d sp = m_lineEnt->startPoint();
	double bearingLength = 0.0;
	if (m_promptSeq == 2) {
        bearingLength = extendDistance();
	}

	    // construct in memory 2dpolyline to trim against
	AcDbPolyline breakPline;
	for (int i=0; i<6; i++) {
		m_breakLine[i].x = m_breakLineOrigin[i].x + m_length;
        m_breakLine[i].y = m_breakLineOrigin[i].y;
		breakPline.addVertexAt(i, AcGePoint2d(m_breakLine[i].x, m_breakLine[i].y));
	}
	    // find the intersect point for the top line
    AcGePoint3d tmpStartPt1(-bearingLength, 0.0, 0.0);
    AcGePoint3d tmpEndPt1(m_length, 0.0, 0.0);

	AcDbLine tmpLine(tmpStartPt1, tmpEndPt1);
	AcGePoint3dArray intPoints;
	Acad::ErrorStatus es = tmpLine.intersectWith(&breakPline, AcDb::kExtendThis, intPoints);
    if (es == Acad::eOk) {
	    m_topPline[0] = intPoints[0];
    }
    else {
	    AEC_ASSERT(es == Acad::eOk);
        m_topPline[0] = tmpEndPt1;
    }

	m_topPline[1] = tmpStartPt1;

        // move the points down
    AcGePoint3d tmpStartPt2(tmpStartPt1);
    AcGePoint3d tmpEndPt2(tmpEndPt1);
    tmpStartPt2.y -= m_chordDepth;
    tmpEndPt2.y -= m_chordDepth;

	m_topPline[2] = tmpStartPt2;

	    // find bottom intersect point
	tmpLine.setStartPoint(tmpStartPt2);
	tmpLine.setEndPoint(tmpEndPt2);
    intPoints.setLogicalLength(0);
	es = tmpLine.intersectWith(&breakPline, AcDb::kExtendThis, intPoints);
    if (es == Acad::eOk) {
	    m_topPline[3] = intPoints[0];
    }
    else {
	    AEC_ASSERT(es == Acad::eOk);
        m_topPline[3] = tmpEndPt2;
    }

	    // now the bottom pline
        // set the x origin to 0.0 since we ignore the bearing dist on the bottom
    tmpStartPt1.x = 0.0;
    tmpStartPt1.y = -m_depth + m_chordDepth;
    tmpEndPt1.y = tmpStartPt1.y;

    tmpStartPt2.x = 0.0;
    tmpStartPt2.y = -m_depth;
    tmpEndPt2.y = tmpStartPt2.y;

	tmpLine.setStartPoint(tmpStartPt2);
	tmpLine.setEndPoint(tmpEndPt2);
	intPoints.setLogicalLength(0);
	es = tmpLine.intersectWith(&breakPline, AcDb::kExtendThis, intPoints);
    if (es == Acad::eOk) {
	    m_bottomPline[0] = intPoints[0];
    }
    else {
	    AEC_ASSERT(es == Acad::eOk);
        m_bottomPline[0] = tmpEndPt2;
    }

	m_bottomPline[1] = tmpStartPt2;
	m_bottomPline[2] = tmpStartPt1;

	    // find bottom intersect point
	tmpLine.setStartPoint(tmpStartPt1);
	tmpLine.setEndPoint(tmpEndPt1);
    intPoints.setLogicalLength(0);
	es = tmpLine.intersectWith(&breakPline, AcDb::kExtendThis, intPoints);
    if (es == Acad::eOk) {
	    m_bottomPline[3] = intPoints[0];
    }
    else {
	    AEC_ASSERT(es == Acad::eOk);
        m_bottomPline[3] = tmpEndPt1;
    }

	if (m_length > m_breakLineFactor)
		calcInterior(&breakPline);
}

/****************************************************************************
**
**  SampEdJigEntCircleJoist::calcInterior
**
**
*************************************/

void						
SampEdJigEntCircleJoist::calcInterior(const AcDbPolyline* breakPline)
{
    freeDbCurves();

        // get lines that we are going to intersect our circles with
    AcDbLine topChordIntrLine(AcGePoint3d(0.0, -m_chordDepth, 0.0), AcGePoint3d(1.0, -m_chordDepth, 0.0));
    AcDbLine bottomChordIntrLine(AcGePoint3d(0.0, -(m_depth - m_chordDepth), 0.0), AcGePoint3d(1.0, -(m_depth - m_chordDepth), 0.0));
    AcDbCircle tmpCirc;

    double midDist = m_depth / 2.0;
    double outerRad = midDist - m_webDia;
    double innerRad = outerRad - m_webDia;

    AcGePoint3d tmpPt(outerRad, -midDist, 0.0);

    int count = static_cast<int>(m_length / (outerRad * 2.0));
    for (int i=0; i<count; i++) {
        tmpCirc.setCenter(tmpPt);
        tmpCirc.setRadius(outerRad);

        trimCircle(&tmpCirc, &topChordIntrLine, &bottomChordIntrLine, breakPline);
        tmpCirc.setRadius(innerRad);
        trimCircle(&tmpCirc, &topChordIntrLine, &bottomChordIntrLine, breakPline);

        tmpPt.x += (outerRad * 2.0);
    }
}

/****************************************************************************
**
**  SampEdJigEntCircleJoist::trimCircle
**      trim off the two sides of the circle where the top and bottom chord
**  lines are
**
**
*************************************/

Acad::ErrorStatus
SampEdJigEntCircleJoist::trimCircle(AcDbCurve* circleToTrim,
                    const AcDbLine* trimLineTop, const AcDbLine* trimLineBottom,
                    const AcDbCurve* breakLine)
{
    AcGePoint3dArray intrPts;
    Acad::ErrorStatus es;

        // if the overall circle intersects the breakLine, trim that piece off
        // before trimming against the chord lines
    // TBD: need to finish this part where it trims the right-most circle against the
    // break line.
    /*if (breakLine != NULL) {
        es = circleToTrim->intersectWith(breakLine, AcDb::kExtendArg, intrPts);
        if ((es == Acad::eOk) && (intrPts.length() > 0)) {
            if (intrPts.length() == 2) {
                double startParam, endParam;
                circleToTrim->getParamAtPoint(intrPts[0], startParam);
                circleToTrim->getParamAtPoint(intrPts[1], endParam);
                if (intrPts[0].y < intrPts[1].y) {
                    double tmpParam = startParam;
                    startParam = endParam;
                    endParam = tmpParam;
                }
                AcGePoint3d pt1, pt2, pt3;
                circleToTrim->getPointAtParam(startParam, pt1);
                circleToTrim->getPointAtParam(endParam, pt3);
                circleToTrim->getPointAtParam((startParam + endParam) / 2.0, pt2);
                AcGeCircArc3d tmpArc(pt1, pt2, pt3);
                AcDbArc keeperCurve(tmpArc.center(), tmpArc.normal(), tmpArc.radius(), tmpArc.startAng(), tmpArc.endAng());
                return trimCircle(&keeperCurve, trimLineTop, trimLineBottom, NULL);
            }
        }
    }*/

    intrPts.setLogicalLength(0);    // reset this
    es = circleToTrim->intersectWith(trimLineTop, AcDb::kExtendArg, intrPts);
    if (es != Acad::eOk) {
        AEC_ASSERT(0);
        return es;
    }

    es = circleToTrim->intersectWith(trimLineBottom, AcDb::kExtendArg, intrPts);
    if (es != Acad::eOk) {
        AEC_ASSERT(0);
        return es;
    }

        // now we need to order the params so that we can cut the circle into consecutive
        // pieces.
    AcGeDoubleArray params;
    double param;
    int len = intrPts.length();
    for (int i=0; i<len; i++) {
        if (circleToTrim->getParamAtPoint(intrPts[i], param) == Acad::eOk) {
            int len2 = params.length();
            bool insertedParam = false;
            for (int j=0;j<len2;j++) {
                if (param < params[j]) {
                    params.insertAt(j, param);
                    insertedParam = true;
                    break;
                }
            }
            if (!insertedParam)
                params.append(param);
        }
        else {
            AEC_ASSERT(0);
        }
    }

        // split the curve into segments where the intersection points were
    AcDbVoidPtrArray curveSegments;
    circleToTrim->getSplitCurves(params, curveSegments);

    len = curveSegments.length();
    if (len == 0)
        m_dbCurvesVisible.append(circleToTrim->clone());
    else {
        AcDbCurve* tmpCurve;
        AcGePoint3d midPt;
        for (int i=0; i<len; i++) {
            tmpCurve = static_cast<AcDbCurve*>(curveSegments[i]);
            midPt = midpoint(tmpCurve);
            if ((midPt.y < trimLineTop->startPoint().y) && (midPt.y > trimLineBottom->startPoint().y))
                m_dbCurvesVisible.append(curveSegments[i]);
            else
                m_dbCurvesHidden.append(curveSegments[i]);
        }
    }

    return Acad::eOk;
}

/****************************************************************************
**
**  SampEdJigEntCircleJoist::midpoint
**
**
*************************************/

AcGePoint3d
SampEdJigEntCircleJoist::midpoint(const AcDbCurve* curve)
{
    Acad::ErrorStatus es;

    double startParam = 0.0;
    es = curve->getStartParam(startParam);
    AEC_ASSERT(es == Acad::eOk);

    double endParam = 1.0;
    es = curve->getEndParam(endParam);
    AEC_ASSERT(es == Acad::eOk);

    AcGePoint3d midPt;
    es = curve->getPointAtParam((startParam + endParam) / 2.0, midPt);

    AEC_ASSERT(es == Acad::eOk);
    return midPt;
}

/****************************************************************************
**
**  SampEdJigEntCircleJoist::setupBreakPoints
**      set this up once at the beginning so we don't have to keep
**  re-calculating it.
**
**
*************************************/

void
SampEdJigEntCircleJoist::setupBreakPoints()
{
	AecDbVarsDwgSetup* dbVars = AecDbVarsDwgSetup::get(AcDb::kForRead, acdbCurDwg());
	m_breakLineFactor = 1.0;
	if (dbVars) {
		m_breakLineFactor = dbVars->dwgScale() * dbVars->textHeight();
		dbVars->close();
	}

	    // make the break line sitting at the origin
	AcGePoint3d tmpPt(0.0, m_breakLineFactor * 2.0, 0.0);

	    // top point
	m_breakLineOrigin[0] = tmpPt;

	tmpPt.y -= (m_breakLineFactor * 2.0) - (m_breakLineFactor - (m_depth / 2.0));
	m_breakLineOrigin[1] = tmpPt;

	tmpPt.y -= m_breakLineFactor / 2.0;
	tmpPt.x -= m_breakLineFactor;
	m_breakLineOrigin[2] = tmpPt;

	tmpPt.x += m_breakLineFactor * 2.0;
	tmpPt.y -= m_breakLineFactor;
	m_breakLineOrigin[3] = tmpPt;

	tmpPt.x -= m_breakLineFactor;
	tmpPt.y -= m_breakLineFactor / 2.0;
	m_breakLineOrigin[4] = tmpPt;

	tmpPt.y -= (m_breakLineFactor * 2.0) - (m_breakLineFactor - (m_depth / 2.0));
	m_breakLineOrigin[5] = tmpPt; 
}

