// $Header: //SandBox/awej/Content/AllUsers/RcpSamples/Source/SampRcp_CPP/Src/SampRcpBulgingMortar.cpp#1 $
// $Author: awej $ $DateTime: 2004/05/05 14:47:54 $ $Change: 11180 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

#include "stdafx.h"
#include "SampRcpBulgingMortar.h"
#include "AecRcpBase/AecRcpIdStr.h"
#include "AecRcpBase/AecRcpJigUtils.h"
#include "AecRcpBase/AecEdJigSurface.h"
#include "AecRcpBase/AecEdJigEntSurface.h"
#include "AecRcpBase/AecRecipeInfoAcc.h"
#include "AecRcpBase/AecRcpCompBldrCurSpace.h"
#include "AecRcpBase/AecSubRecipeInfo.h"
#include "AecRcpBase/AecSubRecipeInfoAcc.h"

/****************************************************************************
**
**  SampRcpBulgingMortar::registerFuncs
**
**
*************************************/

void
SampRcpBulgingMortar::registerFuncs()
{
    addFunc(_DNT("BulgingMortar"),        &mortar);
    addFunc(_DNT("BulgingSubMortar"),     &subMortar);
}

/****************************************************************************
**
**  SampRcpBulgingMortar::mortar
**      draw a mortar joint by itself.  We'll ask for the JointSize and for the
**  Length of the masonary unit since we don't know it ahead of time.
**
**
*************************************/

RcpStatus
SampRcpBulgingMortar::mortar()
{
    AecRecipeInfoAcc rcpInfo;
    if (rcpInfo.isValid() == false)
        return rcpError();
    
    double jointSize;
    bool doHatch;
    if ((rcpInfo.prData()->getFieldValue(_DNT("jointSize"), jointSize) != Acad::eOk) ||
        (rcpInfo.prData()->getFieldValue(_DNT("doHatch"), doHatch) != Acad::eOk)) {
        return rcpError();
    }

        // construct a fake jig ent to draw for the jig.  We'll use the standard
        // Surface Jig which will give us the basic shape dynamically.  If we wanted
        // the exact shape, we would need to write our own derived JigEnt to draw
        // for it.
    AecEdJigEntSurface jigEnt(jointSize);
    jigEnt.setJustification(AecEdJigEntSurface::kLeft);

    AecEdJigSurface jig(&jigEnt);

    AecUiPr::Status stat;
    stat = jig.go();
    if (stat != AecUiPr::kOk)
        return rcpResult(stat);

        // make a CompBldr object to help us do the drawing
    AecRcpCompBldrCurSpace compBldr(rcpInfo);
    if (compBldr.start() != Acad::eOk)
        return rcpError();

        // get the rectangle outline of the surface from the jigEnt
    AcGePoint2dArray pts;
    jigEnt.calculate();
    jigEnt.getEcsOutline(pts);
    compBldr.pushXform(jigEnt.ecsToWcs());

    drawMortar(compBldr, rcpInfo.rcData(), pts, doHatch);

    if (compBldr.endOrAbort() == Acad::eOk)
        return rcpOk();
    else
        return rcpError();
}

/****************************************************************************
**
**  SampRcpBulgingMortar::subMortar
**      This RcpFunc is called as a SubRecipe from another masonary unit Recipe.
**  The contract between the main Recipe and SubRecipe is to pass in the WIDTH
**  of the masonary unit and the HEIGHT of the joint between units.
**
**
*************************************/

RcpStatus
SampRcpBulgingMortar::subMortar()
{
        // NOTE: we get the SubRecipe info here, not the RecipeInfo
    AecSubRecipeInfoAcc rcpInfo;
    if (rcpInfo.isValid() == false)
        return rcpError();

        // need to make sure the main Recipe passed us our Param data
    if (rcpInfo.paramData() == NULL) {
        acutPrintf(_DNT("\nERROR: Recipe \"BulgingSubMortar\" got NULL paramData!"));
        AEC_ASSERT(0);
        return rcpError();
    }

        // paramData() is information passed to us by the main component.  In this
        // case, the Brick has specified what the size of the mortar should be.
    double jointSize, length;
    bool doHatch;
    if ((rcpInfo.paramData()->getFieldValue(_DNT("WIDTH"), length) != Acad::eOk) ||
        (rcpInfo.paramData()->getFieldValue(_DNT("HEIGHT"), jointSize) != Acad::eOk) ||
        (rcpInfo.prData()->getFieldValue(_DNT("doHatch"), doHatch) != Acad::eOk)) {
        return rcpError();
    }

        // continue adding entities to this compBldr
    AecRcpCompBldr* compBldr = rcpInfo.compBldr();

    AcGePoint2d tmpPt;
    AcGePoint2dArray pts;

    tmpPt.y -= jointSize;   // sub-comp origin is upper left, move to lower left corner for consistent starting point
    pts.append(tmpPt);

    tmpPt.x += length;
    pts.append(tmpPt);

    tmpPt.y += jointSize;
    pts.append(tmpPt);

    tmpPt.x -= length;
    pts.append(tmpPt);

    drawMortar(*compBldr, rcpInfo.rcData(), pts, doHatch);

    return rcpOk();
}

/****************************************************************************
**
**  SampRcpBulgingMortar::drawMortar
**      Do the common work of drawing the entities that represent the mortar.
**
**  NOTE: we need to specifically pass in an AecRecipeData object here to get
**  the correct properties from XML.  Normally, you can get it with the following:
**      compBldr.rcpInfo().rcData()
**
**  However, this function is being called for both a main Recipe and a SubRecipe.
**  There are two separate XML files when being called as a SubRecipe and the compBldr
**  contains a pointer to the main Component's XML file, not the Sub-Component's.
**
*************************************/

Acad::ErrorStatus
SampRcpBulgingMortar::drawMortar(AecRcpCompBldr& compBldr, const AecRecipeData* rcData, const AcGePoint2dArray& pts, bool doHatch)
{
    Acad::ErrorStatus es;

        // set any properties for the boundary polyline before adding it to the Db
    AcDbPolyline* pline = new AcDbPolyline;
    compBldr.setToDefaultProps(pline);
    rcData->getItem_EntityProps(AecRcpIdStr::boundary, pline);

        // we got a rectangular outline, but we'll add a bulge at each end to 
        // make the mortar look "bulging"
    AcGeDoubleArray bulges;
    bulges.append(0.0);
    bulges.append(1.0);
    bulges.append(0.0);
    bulges.append(1.0);
    es = compBldr.addPolyline(pline, pts, bulges, true);    // takes responsibility for pline

    if ((es == Acad::eOk) && doHatch) {
        AcDbHatch* hatch = NULL;
        if (rcData->getItem_Hatch(AecRcpIdStr::infill, hatch) == Acad::eOk) {
            compBldr.setToDefaultProps(hatch);
            rcData->getItem_EntityProps(AecRcpIdStr::infill, hatch);
            es = compBldr.addHatchToPrevBoundary(hatch, true);
        }
    }

    return es;
}
