// $Header: //SandBox/awej/Content/AllUsers/RcpSamples/Source/SampRcp_CPP/Src/SampRcpCircleJoists.cpp#1 $
// $Author: awej $ $DateTime: 2004/05/05 14:47:54 $ $Change: 11180 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

#include "stdafx.h"
#include "SampRcpCircleJoists.h"
#include "SampEdJigEntCircleJoist.h"
#include "SampUtils.h"
#include "AecRcpBase/AecEdJigLineExtend.h"
#include "AecRcpBase/AecRcpUtils.h"
#include "AecRcpBase/AecRecipeInfoAcc.h"
#include "AecRcpBase/AecRcpIdStr.h"
#include "AecRcpBase/AecRcpJigUtils.h"
#include "AecRcpBase/AecRcpCompBldrCurSpace.h"

/****************************************************************************
**
**  SampRcpCircleJoist::registerFuncs
**
**
*************************************/

void
SampRcpCircleJoist::registerFuncs()
{
    addFunc(_DNT("CircleJoistSection"),		&section);
    addFunc(_DNT("CircleJoistPlan"),	    &plan);
    addFunc(_DNT("CircleJoistElevation"),   &elevation);
}

/****************************************************************************
**
**  SampRcpCircleJoist::section
**
**
*************************************/

RcpStatus
SampRcpCircleJoist::section()
{
    AecRecipeInfoAcc rcpInfo;
    if (rcpInfo.isValid() == false)
        return rcpError();

	double depth, chordWidth, chordDepth, dia;
	AcDbDatabase* db = rcpInfo.database();

	if ((rcpInfo.dbData()->getFieldValueUnitDep(_DNT("DEPTH"), depth, db) != Acad::eOk) ||
		(rcpInfo.dbData()->getFieldValueUnitDep(_DNT("CHORD_W"), chordWidth, db) != Acad::eOk) ||
		(rcpInfo.dbData()->getFieldValueUnitDep(_DNT("CHORD_D"), chordDepth, db) != Acad::eOk) ||
		(rcpInfo.dbData()->getFieldValueUnitDep(_DNT("WEB_DIA"), dia, db) != Acad::eOk)) {
        return rcpError();
	}
        // build the block
    AecRcpCompBldrBlkDef compBldr(rcpInfo);
    if (compBldr.start() != Acad::eOk)
        return rcpError();

	    // make the entities that make up the block
	Acad::ErrorStatus es = createSectionEnts(compBldr, depth, chordWidth, chordDepth, dia);
	if (es != Acad::eOk)
		return rcpError();

        // stamp out the built block
    if (compBldr.endOrAbort() == Acad::eOk)
        return rcpResult(AecRcpJigUtils::stampIt(compBldr));
    else
        return rcpError();
}

/****************************************************************************
**
**  SampRcpCircleJoist::createSectionEnts
**
**
*************************************/

Acad::ErrorStatus    
SampRcpCircleJoist::createSectionEnts(AecRcpCompBldrBlkDef& compBldr,
					double depth, double chordWidth, double chordDepth, double dia)

{
	    // starting with the middle/top of the joist as the origin point, make a
        // upside down u-shape (chordDepth on each leg of the U)
    AcGePoint2dArray pts;
	AcGePoint2d tmpPt = AcGePoint2d::kOrigin;

	tmpPt.x -= chordDepth + (dia / 2.0);
    pts.append(tmpPt);                      // upper left corner

    tmpPt.y -= chordWidth;
    pts.append(tmpPt);                      // lower left outside corner

    tmpPt.x += chordDepth;
    pts.append(tmpPt);                      // lower left inside corner

    tmpPt.y += chordWidth - dia;
    pts.append(tmpPt);                      // inside left of U

    tmpPt.x += dia;
    pts.append(tmpPt);                      // inside right of U

    tmpPt.y -= chordWidth - dia;
    pts.append(tmpPt);                      // lower right inside corner

    tmpPt.x += chordDepth;
    pts.append(tmpPt);                      // lower right outside corner

    tmpPt.y += chordWidth;
    pts.append(tmpPt);                      // upper right corner

	AcDbPolyline* pline1 = new AcDbPolyline;
    compBldr.setToDefaultProps(pline1);
    compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::boundary, pline1);
    compBldr.addPolyline(pline1, pts, true);

        // mirror those points to get the bottom chord
    AcGeMatrix3d mat;
    mat.setToMirroring(AcGeLine3d(AcGePoint3d(-1.0, -(depth/2.0), 0.0), AcGePoint3d(1.0, -(depth/2.0), 0.0)));
    compBldr.pushXform(mat);

    AcDbPolyline* pline2 = new AcDbPolyline;
    pline2->setPropertiesFrom(pline1);
    compBldr.addPolyline(pline2, pts, true);
    compBldr.popXform();

        // now draw circle webbing (from end view)
    pts.setLogicalLength(0);
    tmpPt.x = -(dia / 2.0);
    tmpPt.y = -(dia * 1.5);
    pts.append(tmpPt);

    tmpPt.y -= depth - (dia * 3.0);
    pts.append(tmpPt);

    tmpPt.x += dia;
    pts.append(tmpPt);

    tmpPt.y += depth - (dia * 3.0);
    pts.append(tmpPt);

    AcGeDoubleArray bulges;
    bulges.append(0.0);
    bulges.append(1.0);
    bulges.append(0.0);
    bulges.append(1.0);

	AcDbPolyline* pline3 = new AcDbPolyline;
    compBldr.setToDefaultProps(pline3);
    compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::web, pline3);
    compBldr.addPolyline(pline3, pts, bulges, true);

	return Acad::eOk;
}

/****************************************************************************
**
**  SampRcpCircleJoist::plan
**      It is a common recipe to draw a series of horizontal lines.  We just
**  need to obtain our specific information and then pass it on to a common
**  utility function to do the work.
**
**
*************************************/

RcpStatus
SampRcpCircleJoist::plan()
{
	AecRecipeInfoAcc rcpInfo;
	if (rcpInfo.isValid() == false)
		return rcpError();

	double width, dia;
	bool hideWeb;
	AcDbDatabase* db = rcpInfo.database();

	if ((rcpInfo.dbData()->getFieldValueUnitDep(_DNT("CHORD_D"), width, db) != Acad::eOk) ||
		(rcpInfo.dbData()->getFieldValueUnitDep(_DNT("WEB_DIA"), dia, db) != Acad::eOk) ||
		(rcpInfo.prData()->getFieldValue(_DNT("hideWeb"), hideWeb) != Acad::eOk)) {
        return rcpError();
    }

    AcGeDoubleArray offsets;
    AecRmCStringArray compIdStrs;

    AecRmCString str;
    if (hideWeb)
        str = AecRcpIdStr::hidden;
    else
        str = _DNT("");

	double offset = dia / 2.0;
	offsets.append(offset);
	offsets.append(-offset);
	compIdStrs.Add(str);
	compIdStrs.Add(str);

    return SampUtils::createPlanOrElev(rcpInfo, ((width * 2.0) + dia), offsets, compIdStrs, SampUtils::kCenter);
}

/****************************************************************************
**
**  SampRcpCircleJoist::elevation
**
**
*************************************/

RcpStatus
SampRcpCircleJoist::elevation()
{

    AecRecipeInfoAcc rcpInfo;
    if (rcpInfo.isValid() == false)
        return rcpError();

	double depth, chordWidth, dia;
	AcDbDatabase* db = rcpInfo.database();

	if ((rcpInfo.dbData()->getFieldValueUnitDep(_DNT("DEPTH"), depth, db) != Acad::eOk) ||
		(rcpInfo.dbData()->getFieldValueUnitDep(_DNT("CHORD_W"), chordWidth, db) != Acad::eOk) ||
		(rcpInfo.dbData()->getFieldValueUnitDep(_DNT("WEB_DIA"), dia, db) != Acad::eOk)) {
        return rcpError();
	}

    bool showHidden = true;
    rcpInfo.prData()->getFieldValue(_DNT("showHidden"), showHidden, false);

        // plug our JigEnt into the standard AecEdJigLineExtend Jig.  The Jig will
        // handle all the user control, but our JigEnt will get to do the visual
        // feedback.
	SampEdJigEntCircleJoist jigEnt(depth, chordWidth, dia);
	AecEdJigLineExtend jig(&jigEnt);
	jig.setExtendPrompt(_DNT("Bearing length:"));

	AecUiPr::Status stat = jig.go();
	if (stat != AecUiPr::kOk) { // operation has been canceled
		return rcpResult(stat);
	}

	    // build the block
	AecRcpCompBldrBlkDef compBldr(rcpInfo);
	if (compBldr.start() != Acad::eOk)
		return rcpError();

	jigEnt.calculate();
	    // get the rectangle outline of the surface from the jigEnt
	AcGePoint2dArray pts;
	jigEnt.getEcsOutline(pts, SampEdJigEntCircleJoist::kBreakLine);

	    // set any properties for the boundary polyline before adding it to the Db
	AcDbPolyline* plineBreak = new AcDbPolyline;
	compBldr.setToDefaultProps(plineBreak);
	rcpInfo.rcData()->getItem_EntityProps(AecRcpIdStr::breakline, plineBreak);
	Acad::ErrorStatus es = compBldr.addPolyline(plineBreak, pts, false);

	AcDbPolyline* plineTop = new AcDbPolyline;
	compBldr.setToDefaultProps(plineTop);
	rcpInfo.rcData()->getItem_EntityProps(AecRcpIdStr::boundary, plineTop);
	jigEnt.getEcsOutline(pts, SampEdJigEntCircleJoist::kTopPline);
	es = compBldr.addPolyline(plineTop, pts, false);

	AcDbPolyline* plineBottom = new AcDbPolyline;
    plineBottom->setPropertiesFrom(plineTop);
	jigEnt.getEcsOutline(pts, SampEdJigEntCircleJoist::kBottomPline);
	es = compBldr.addPolyline(plineBottom, pts, false);

        // get a template for properties since we have many arcs and
        // we don't want to have to look up the value each time
    AcDbLine tmplPropsLine;
	compBldr.setToDefaultProps(&tmplPropsLine);
	rcpInfo.rcData()->getItem_EntityProps(AecRcpIdStr::web, &tmplPropsLine);

    AcDbCurve* tmpCurve;
	AcDbVoidPtrArray& webEntsVisible = jigEnt.curvesVisible();
	int len = webEntsVisible.length();
    for (int i=0; i<len; i++) {
		tmpCurve = static_cast<AcDbCurve*>(webEntsVisible[i]);
        tmpCurve->setPropertiesFrom(&tmplPropsLine);
		compBldr.addToDb(tmpCurve);
	}
    webEntsVisible.setLogicalLength(0); // we took control of these

    if (showHidden) {
	    compBldr.setToDefaultProps(&tmplPropsLine);
	    rcpInfo.rcData()->getItem_EntityProps(AecRcpIdStr::hidden, &tmplPropsLine);

	    AcDbVoidPtrArray& webEntsHidden = jigEnt.curvesHidden();
	    len = webEntsHidden.length();
        for (int i=0; i<len; i++) {
		    tmpCurve = static_cast<AcDbCurve*>(webEntsHidden[i]);
            tmpCurve->setPropertiesFrom(&tmplPropsLine);
		    compBldr.addToDb(tmpCurve);
	    }
        webEntsHidden.setLogicalLength(0);  // we took control of these
    }

        // when the user picked the points to define the start and end of the Joist,
        // they already told us where they wanted it.  However, we want to make a block
        // that has the origin in a reasonable spot.  The utility function placeBlockWhereCreated()
        // will take the BlockDef defined in the CompBldr and create an AcDbBlockReference
        // at the same spot where they picked the insertion point of the Joist already.
    if (compBldr.endOrAbort() == Acad::eOk) {
        if (AecRcpJigUtils::placeBlockWhereCreated(compBldr, jigEnt.ecsToWcs()) == Acad::eOk)
            return rcpOk();
        else
            return rcpError();
    }
    else
        return rcpError();
}




