// $Header: //SandBox/awej/Content/AllUsers/RcpSamples/Source/SampRcp_CPP/Src/SampRcpCobblestone.cpp#1 $
// $Author: awej $ $DateTime: 2004/05/05 14:47:54 $ $Change: 11180 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

#include "stdafx.h"
#include "SampRcpCobblestone.h"
#include "SampUtils.h"
#include "AecRcpBase/AecRecipeInfoAcc.h"
#include "AecRcpBase/AecRcpIdStr.h"
#include "AecRcpBase/AecRcpJigUtils.h"

/****************************************************************************
**
**  SampRcpCobblestone::registerFuncs
**      This is where we register our recipe functions.  The name registered
**  is the name that will be referenced in the XML Recipe Spec.  The names
**  occur in our Apps namespace, so they only have to be unique within our app.
**
**
*************************************/

void
SampRcpCobblestone::registerFuncs()
{
    addFunc(_DNT("CobblestoneSection"),    &section);
}

/****************************************************************************
**
**  SampRcpCobblestone::section
**      Draw a section view of our cobblestone.  We just need to draw a simple
**  rectangle.  Then we'll pass it off to some common utility functions that know
**  how to take an array of points and bulges and make a section profile out of it.
**
**
*************************************/

RcpStatus
SampRcpCobblestone::section()
{
        // get the Recipe Info Accessor object
    AecRecipeInfoAcc rcpInfo;
    if (rcpInfo.isValid() == false)
        return rcpError();
    
    double width, thick;
    bool doHatch;

        // retrieve the values we need:  dbData() = values from the database,
        // prData() = values from prompts (Property Palette), rcData() = RecipeSpec data from XML
        // NOTE: distance values are usually unit-dependent (thus the "UnitDep" version of the function)
    AcDbDatabase* db = rcpInfo.database();
    if ((rcpInfo.dbData()->getFieldValueUnitDep(_DNT("THICK"), thick, db) != Acad::eOk) ||
        (rcpInfo.dbData()->getFieldValueUnitDep(_DNT("WIDTH"), width, db) != Acad::eOk) ||
        (rcpInfo.prData()->getFieldValue(_DNT("doHatch"), doHatch) != Acad::eOk)) {
        return rcpError();
    }

        // build the block adding objects into a Component Builder
    AecRcpCompBldrBlkDef compBldr(rcpInfo);
    if (compBldr.start() != Acad::eOk)
        return rcpError();

        // draw a rectangle for the cobblestone
    AcGePoint2d tmpPt;
    AcGePoint2dArray pts;
    AcGeDoubleArray bulges; // we won't have any

    pts.append(tmpPt);      // lower left corner (at origin)

    tmpPt.x += width;
    pts.append(tmpPt);      // lower right corner

    tmpPt.y += thick;
    pts.append(tmpPt);      // top right corner

    tmpPt.x -= width;
    pts.append(tmpPt);      // top left corner

        // the values above *MUST* be present or the function will return an error.
        // Just for demonstration, we'll optionally get the next two parameters.  If they
        // don't exist in the XML file, then it will return silently (due to the "false" param)
        // and use the default value that we've set.
    bool coarseOutline = true;
    compBldr.rcpInfo().prData()->getFieldValue(_DNT("coarseOutline"), coarseOutline, false);

        // make the block
    Acad::ErrorStatus es;
    if (coarseOutline)
        es = SampUtils::createSquiggledSection(compBldr, pts, bulges);
    else
        es = SampUtils::createSection(compBldr, pts, bulges);

        // stamp it out
    if (es == Acad::eOk)
        return rcpResult(AecRcpJigUtils::linearArrayIt(compBldr, width, 0.0));
    else
        return rcpError();
}
