// $Header: //SandBox/awej/Content/AllUsers/RcpSamples/Source/SampRcp_CPP/Src/SampRcpCoredBrick.cpp#1 $
// $Author: awej $ $DateTime: 2004/05/05 14:47:54 $ $Change: 11180 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

#include "stdafx.h"
#include "SampRcpCoredBrick.h"
#include "AecRcpBase/AecRecipeInfoAcc.h"
#include "AecRcpBase/AecRcpIdStr.h"
#include "AecRcpBase/AecRcpJigUtils.h"
//#include "AecUiBase/AecSysVarNode.h"
#include "AecUiBase/AecuUtils.h"

/****************************************************************************
**
**  SampRcpCoredBrick::registerFuncs
**      This is where we register our recipe functions.  The name registered
**  is the name that will be referenced in the XML Recipe Spec.  The names
**  occur in our Apps namespace, so they only have to be unique within our app.
**
**
*************************************/

void
SampRcpCoredBrick::registerFuncs()
{
    addFunc(_DNT("CoredBrickSection"),    &section);
    addFunc(_DNT("CoredBrickPlan"),       &plan);
    addFunc(_DNT("CoredBrickElevation"),  &elevation);
}

/****************************************************************************
**
**  SampRcpCoredBrick::section
**      Draw a section view of our brick.  The brick recipes that come with
**  ADT allow for different orientations of the brick.  For this example, we
**  we simplify it to a single orientation.  If you think about the normal 
**  orientation of a brick, WIDTH = YAxis, LENGTH = XAxis, and THICK = ZAxis.
**  For Section view, we are looking at the end of the brick.
**
**
*************************************/

RcpStatus
SampRcpCoredBrick::section()
{
        // get the Recipe Info Accessor object
    AecRecipeInfoAcc rcpInfo;
    if (rcpInfo.isValid() == false)
        return rcpError();
    
    double width, thick, joint, coreRad;
    bool doHatch;

        // retrieve the values we need:  dbData() = values from the database,
        // prData() = values from prompts (Property Palette), rcData() = RecipeSpec data from XML
        // NOTE: distance values are usually unit-dependent (thus the "UnitDep" version of the function)
    AcDbDatabase* db = rcpInfo.database();
    if ((rcpInfo.dbData()->getFieldValueUnitDep(_DNT("THICK"), thick, db) != Acad::eOk) ||
        (rcpInfo.dbData()->getFieldValueUnitDep(_DNT("WIDTH"), width, db) != Acad::eOk) ||
        (rcpInfo.dbData()->getFieldValueUnitDep(_DNT("CORE_RADIUS"), coreRad, db) != Acad::eOk) ||
        (rcpInfo.prData()->getFieldValueUnitDep(_DNT("jointSize"), joint, db) != Acad::eOk) ||
        (rcpInfo.prData()->getFieldValue(_DNT("doHatch"), doHatch) != Acad::eOk)) {
        return rcpError();
    }

        // the values above *MUST* be present or the function will return an error.
        // Just for demonstration, we'll optionally get the next two parameters.  If they
        // don't exist in the XML file, then it will return silently (due to the "false" param)
        // and use the default value that we've set.
    bool showCores = true;
    rcpInfo.prData()->getFieldValue(_DNT("showCores"), showCores, false);

    bool showMortar = false;
    rcpInfo.prData()->getFieldValue(_DNT("showMortar"), showMortar, false);

        // build the block adding objects into a Component Builder
    AecRcpCompBldrBlkDef compBldr(rcpInfo);
    if (compBldr.start() != Acad::eOk)
        return rcpError();

        // we want the origin of our block to be at the bottom of the mortar
        // joint.  So, we will push an Xform into the CompBldr so that the
        // brick can assume its ECS starts at the Origin (when it really starts Y=joint).
    AcGeMatrix3d xform;
    xform.setToTranslation(AcGeVector3d(0.0, joint, 0.0));
    compBldr.pushXform(xform);

        // make the entities that make up the block
    Acad::ErrorStatus es;
    if (showCores)
        es = createCoredBrickSection(compBldr, width, thick, doHatch, coreRad);
    else
        es = createSimpleBrick(compBldr, width, thick, doHatch);

    if (es != Acad::eOk)
        return rcpError();

        // if the mortar sub-component is turned on, call the sub-recipe.
        // The sub-recipe "mortar" is the actual name you'll see
        // in XML as <SubComponent idStr="mortar" ...>, which will then 
        // list the name of the subRecipe to use from the selected sub-component's
        // XML file.
    if (showMortar) {
        AecRcpDataGroup paramData(AecRcpDataGroup::kParamData);
        paramData.addField(_DNT("WIDTH"), width);
        paramData.addField(_DNT("HEIGHT"), joint);
        rcpInfo.rcInfo()->runSubRecipe(_DNT("mortar"), &compBldr, &paramData);
    }

    compBldr.popXform();

        // array the block
    if (compBldr.endOrAbort() == Acad::eOk)
        return rcpResult(AecRcpJigUtils::linearArrayIt(compBldr, thick+joint, 0.0));
    else
        return rcpError();
}

/****************************************************************************
**
**  SampRcpCoredBrick::plan
**      Our plan view will be looking down on top of the brick, with all the
**  cores showing.
**
*************************************/

RcpStatus
SampRcpCoredBrick::plan()
{
    AecRecipeInfoAcc rcpInfo;
    if (rcpInfo.isValid() == false)
        return rcpError();
    
    double length, width, joint, coreRad;
    bool doHatch;
    long numCores;

    AcDbDatabase* db = rcpInfo.database();
    if ((rcpInfo.dbData()->getFieldValueUnitDep(_DNT("LENGTH"), length, db) != Acad::eOk) ||
        (rcpInfo.dbData()->getFieldValueUnitDep(_DNT("WIDTH"), width, db) != Acad::eOk) ||
        (rcpInfo.dbData()->getFieldValueUnitDep(_DNT("CORE_RADIUS"), coreRad, db) != Acad::eOk) ||
        (rcpInfo.dbData()->getFieldValue(_DNT("NUM_CORES"), numCores) != Acad::eOk) ||
        (rcpInfo.prData()->getFieldValueUnitDep(_DNT("jointSize"), joint, db) != Acad::eOk) ||
        (rcpInfo.prData()->getFieldValue(_DNT("doHatch"), doHatch) != Acad::eOk)) {
        return rcpError();
    }

    bool showCores = true;
    rcpInfo.prData()->getFieldValue(_DNT("showCores"), showCores, false);

    bool showMortar = false;
    rcpInfo.prData()->getFieldValue(_DNT("showMortar"), showMortar, false);

        // build the block
    AecRcpCompBldrBlkDef compBldr(rcpInfo);
    if (compBldr.start() != Acad::eOk)
        return rcpError();

        // we want the brick to draw as if it was standing, but when we lay it out
        // we want it tipped over.  By pushing in an Xform, the brick doesn't have to
        // know what orientation it is and can just draw as normal.
    AcGeMatrix3d xform;
    xform.setCoordSystem(AcGePoint3d(joint, width, 0.0), -AcGeVector3d::kYAxis, AcGeVector3d::kXAxis, AcGeVector3d::kZAxis);
    compBldr.pushXform(xform);

        // make the entities that make up the block
    Acad::ErrorStatus es;
    if (showCores)
        es = createCoredBrickPlan(compBldr, width, length, doHatch, coreRad, numCores);
    else
        es = createSimpleBrick(compBldr, width, length, doHatch);

    if (es != Acad::eOk)
        return rcpError();

        // call sub-component to draw the mortar (see notes in ::section() recipe).
    if (showMortar) {
        AecRcpDataGroup paramData(AecRcpDataGroup::kParamData);
        paramData.addField(_DNT("WIDTH"), width);
        paramData.addField(_DNT("HEIGHT"), joint);
        rcpInfo.rcInfo()->runSubRecipe(_DNT("mortar"), &compBldr, &paramData);
    }

    compBldr.popXform();

        // array the block
    if (compBldr.endOrAbort() == Acad::eOk)
        return rcpResult(AecRcpJigUtils::linearArrayIt(compBldr, length+joint, 0.0));
    else
        return rcpError();
}

/****************************************************************************
**
**  SampRcpCoredBrick::elevation
**
**
*************************************/

RcpStatus
SampRcpCoredBrick::elevation()
{
    AecRecipeInfoAcc rcpInfo;
    if (rcpInfo.isValid() == false)
        return rcpError();
    
    double length, thick, joint;

    AcDbDatabase* db = rcpInfo.database();
    if ((rcpInfo.dbData()->getFieldValueUnitDep(_DNT("LENGTH"), length, db) != Acad::eOk) ||
        (rcpInfo.dbData()->getFieldValueUnitDep(_DNT("THICK"), thick, db) != Acad::eOk) ||
        (rcpInfo.prData()->getFieldValueUnitDep(_DNT("jointSize"), joint, db) != Acad::eOk)) {
        return rcpError();
    }

        // build the block
    AecRcpCompBldrBlkDef compBldr(rcpInfo);
    if (compBldr.start() != Acad::eOk)
        return rcpError();

        // set up translation to account for the mortar
    AcGeMatrix3d xform;
    xform.setToTranslation(AcGeVector3d(0.0, joint, 0.0));
    compBldr.pushXform(xform);

        // make the entities that make up the block (no hatching)
    Acad::ErrorStatus es;
    es = createSimpleBrick(compBldr, length, thick, false);
    if (es != Acad::eOk) {
        compBldr.abort();
        return rcpError();
    }

    compBldr.popXform();

        // Set the new snap units, automatically restored once the object goes out of scope.
        // This allows the user to easily stack them next to or on top of each other.
    AcGePoint2d newSnapUnits((length + joint) / 6.0, (thick + joint));
    //AecSysVarStack sysVarStack;
    //sysVarStack.push(_DNT("osmode"), 0);
    //sysVarStack.push(_DNT("snapunit"), newSnapUnits);

        // Hmmmm... that class doesn't appear to be exported.  We'll have to do it the
        // old-fashioned way!  :-(
    int tmpOsmode = 0;
    Aecu::getSysVar(_DNT("osmode"), tmpOsmode);
    AcGePoint2d tmpSnapunit;
    Aecu::getSysVar(_DNT("snapunit"), tmpSnapunit);

        // stamp out the built block
    if (compBldr.endOrAbort() == Acad::eOk) {
        Aecu::setSysVar(_DNT("osmode"), 0);
        Aecu::setSysVar(_DNT("snapunit"), newSnapUnits);

        AecUiPr::Status stat = AecRcpJigUtils::stampIt(compBldr);

        Aecu::setSysVar(_DNT("osmode"), tmpOsmode);
        Aecu::setSysVar(_DNT("snapunit"), tmpSnapunit);

        return rcpResult(stat);
    }
    else
        return rcpError();
}

/****************************************************************************
**
**  SampRcpCoredBrick::createSimpleBrick
**      This just creates a simple brick without showing the cores.  This is
**  to demonstrate that a simple boundary with Hatch is relatively easy to do.
**  See SampRcpCoredBrick::createCoredBrickSection() for a more complex hatch boundary.
**
**
*************************************/

Acad::ErrorStatus  
SampRcpCoredBrick::createSimpleBrick(AecRcpCompBldr& compBldr, double xDist, double yDist, bool doHatch)
{
    AcGePoint2d tmpPt;
    AcGePoint2dArray pts;

    pts.append(tmpPt);  // lower left corner

    tmpPt.x += xDist;
    pts.append(tmpPt);  // lower right corner

    tmpPt.y += yDist;
    pts.append(tmpPt);  // upper right corner

    tmpPt.x = 0.0;
    pts.append(tmpPt);  // upper left corner

    AcDbPolyline* pline = new AcDbPolyline;
    compBldr.setToDefaultProps(pline);
    compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::boundary, pline);

    Acad::ErrorStatus es;
    es = compBldr.addPolyline(pline, pts, true);
    if ((es == Acad::eOk) && doHatch) {
        AcDbHatch* hatch = NULL;
        if (compBldr.rcpInfo().rcData()->getItem_Hatch(AecRcpIdStr::infill, hatch) == Acad::eOk) {
            compBldr.setToDefaultProps(hatch);
            compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::infill, hatch);
            es = compBldr.addHatchToPrevBoundary(hatch, true);
        }
    }

    return es;
}

/****************************************************************************
**
**  SampRcpCoredBrick::createCoredBrickSection
**      This is a section cut of the brick from the end view.  Therefore, no
**  matter how many cores there are, we will only see one of them (which we
**  will represent with a line on each side of the core and no hatch between).
**
**
*************************************/

Acad::ErrorStatus  
SampRcpCoredBrick::createCoredBrickSection(AecRcpCompBldr& compBldr, double width, double height, bool doHatch, double coreRadius)
{
        // first create the boundary
    createSimpleBrick(compBldr, width, height, false);  // we don't want the hatch drawn for this boundary

        // now create the lines to represent the row of cores (there will only be one from our end view)
    double halfWidth = width / 2.0;

    AcDbLine* line1 = new AcDbLine(AcGePoint3d(halfWidth - coreRadius, 0.0, 0.0),
                                   AcGePoint3d(halfWidth - coreRadius, height, 0.0));
    compBldr.setToDefaultProps(line1);
    compBldr.rcpInfo().rcData()->getItem_EntityProps(_DNT("cores"), line1);
    compBldr.addToDb(line1);

        // when creating mulitple entities as part of the same sub-part, it is advantageous
        // not to return to the XML to retrieve the same set of properties (especially if you
        // have a lot of linework).  So for the second line, we can just quickly copy the properties
        // from the first one.
    AcDbLine* line2 = new AcDbLine(AcGePoint3d(halfWidth + coreRadius, 0.0, 0.0),
                                   AcGePoint3d(halfWidth + coreRadius, height, 0.0));
    line2->setPropertiesFrom(line1);
    compBldr.addToDb(line2);

        // now we need to make two separate hatch boundaries on each side of the core
    if (doHatch) {
        AcGePoint2d tmpPt;
        AcGePoint2dArray pts;

        pts.append(tmpPt);  // start at lower left (origin or our ECS)
        
        tmpPt.x += (halfWidth - coreRadius);
        pts.append(tmpPt);  // bottom right of left boundary

        tmpPt.y += height;
        pts.append(tmpPt);  // top right of left boundary

        tmpPt.x = 0.0;      
        pts.append(tmpPt);  // top left of left boundary

        pts.append(pts.first());    // have to explictly close manual boundaries

        AcGeDoubleArray bulges;     // we don't have any, so we can pass an empty array

        AcDbHatch* hatch1 = NULL;
        if (compBldr.rcpInfo().rcData()->getItem_Hatch(AecRcpIdStr::infill, hatch1) == Acad::eOk) {
            compBldr.setToDefaultProps(hatch1);
            compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::infill, hatch1);
            compBldr.addHatchWithBoundary(hatch1, pts, bulges);
        }

            // now create the right side boundary by shifting all the points over in the X direction
        int len = pts.length();
        for (int i=0; i<len; i++) {
            pts[i].x += (halfWidth + coreRadius);
        }

        AcDbHatch* hatch2 = NULL;
        if (compBldr.rcpInfo().rcData()->getItem_Hatch(AecRcpIdStr::infill, hatch2) == Acad::eOk) {
            compBldr.setToDefaultProps(hatch2);
            compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::infill, hatch2);
            compBldr.addHatchWithBoundary(hatch2, pts, bulges);
        }
    }

    return Acad::eOk;
}

/****************************************************************************
**
**  SampRcpCoredBrick::createCoredBrickPlan
**      This view is looking down on top of the brick.  In this case, we can
**  see all the cores, and they all contribute to the hatch boundary.
**
**
*************************************/

Acad::ErrorStatus  
SampRcpCoredBrick::createCoredBrickPlan(AecRcpCompBldr& compBldr, double xDist, double yDist,
                        bool doHatch, double coreRadius, long numCores)
{
    Acad::ErrorStatus es;
    AcDbObjectIdArray boundaryEnts;     // keep track of things that were a hatch boundary

    AcGePoint2dArray pts;
	AcGePoint2d tmpPt;

    pts.append(tmpPt);  // lower left corner

    tmpPt.x += xDist;
    pts.append(tmpPt);  // lower right corner

    tmpPt.y += yDist;
    pts.append(tmpPt);  // upper right corner

    tmpPt.x = 0.0;
    pts.append(tmpPt);  // upper left corner

	AcDbPolyline* pline = new AcDbPolyline;
    compBldr.setToDefaultProps(pline);
    compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::boundary, pline);
    es = compBldr.addPolyline(pline, pts, true);
    if (es != Acad::eOk)
        return es;

    boundaryEnts.append(pline->objectId());     // record this to use for hatch boundary

        // now find out how long the row of circles will be so we can center them
    double holeDist = coreRadius * 3.0;    // distance between center points
    double totalHoleDist = holeDist * (numCores - 1);
    tmpPt.x = (xDist / 2.0);
    tmpPt.y = (yDist / 2.0) - (totalHoleDist / 2.0);   // center holes in center of brick

        // make a template circle so we don't have to keep looking properties up over
        // and over again from the XML.
    AcDbCircle* templateCirc = new AcDbCircle;
    templateCirc->setRadius(coreRadius);
    compBldr.setToDefaultProps(templateCirc);
    compBldr.rcpInfo().rcData()->getItem_EntityProps(_DNT("cores"), templateCirc);

	for (int i=0; i<numCores; i++) {
		AcDbCircle* circle = AcDbCircle::cast(templateCirc->clone());
		circle->setCenter(AcGePoint3d(tmpPt.x, tmpPt.y, 0.0));
        es = compBldr.addToDb(circle);
        if (es == Acad::eOk)
            boundaryEnts.append(circle->objectId());

        tmpPt.y += holeDist;
	}

    delete templateCirc;

    if (doHatch) {
        AcDbHatch* hatch = NULL;
        if (compBldr.rcpInfo().rcData()->getItem_Hatch(AecRcpIdStr::infill, hatch) == Acad::eOk) {
            compBldr.setToDefaultProps(hatch);
            compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::infill, hatch);
            es = compBldr.addHatchToBoundaries(hatch, boundaryEnts, true, AcDbHatch::kNormal);
        }
    }

    return Acad::eOk;
}