// $Header: //SandBox/awej/Content/AllUsers/RcpSamples/Source/SampRcp_CPP/Src/SampUtils.cpp#1 $
// $Author: awej $ $DateTime: 2004/05/05 14:47:54 $ $Change: 11180 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#include "stdafx.h"
#include "SampUtils.h"
#include "AecRcpBase/AecRcpJigUtils.h"
#include "AecRcpBase/AecRcpIdStr.h"
#include "AecRcpBase/AecRecipeUtils.h"
#include "AecRcpBase/AecRcpUtils.h"
#include "AecRcpBase/AecRecipeInfoAcc.h"
#include "AecRcpBase/AecEdJigSurface.h"
#include "AecRcpBase/AecEdJigEntSurfaceHlines.h"
#include "AecRcpBase/AecRcpCompBldrCurSpace.h"

/****************************************************************************
**
**  SampUtils::createSection
**      there are many "section" recipes that consist of a Polyline boundary
**  and a Hatch infill.  Given a Point and Bulge array, the common work can
**  be abstracted out.
**
*************************************/

Acad::ErrorStatus
SampUtils::createSection(AecRcpCompBldrBlkDef& compBldr, const AcGePoint2dArray& pts,
                            const AcGeDoubleArray& bulges)
{
        // see if they specified a Hatch or not
    bool doHatch = false;
    compBldr.rcpInfo().prData()->getFieldValue(_DNT("doHatch"), doHatch, false);

    AcDbPolyline* pline = new AcDbPolyline;
    compBldr.setToDefaultProps(pline);
    compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::boundary, pline);
    
    Acad::ErrorStatus es;
    if (bulges.isEmpty())
        es = compBldr.addPolyline(pline, pts, true);
    else
        es = compBldr.addPolyline(pline, pts, bulges, true);

    if (es == Acad::eOk) {
        if (doHatch) {
            AcDbHatch* hatch = NULL;
            if (compBldr.rcpInfo().rcData()->getItem_Hatch(AecRcpIdStr::infill, hatch) == Acad::eOk) {
                compBldr.setToDefaultProps(hatch);
                compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::infill, hatch);
                es = compBldr.addHatchToPrevBoundary(hatch, true);
            }
        }

        return compBldr.endOrAbort();     // finish our block def
    }
    else {
        return compBldr.abort();
    }
}

/****************************************************************************
**
**  SampUtils::createSquiggledSection
**      Similar to the above function except that we want to make squiggled
**  edges to our boundary.
**
**
*************************************/

Acad::ErrorStatus
SampUtils::createSquiggledSection(AecRcpCompBldrBlkDef& compBldr,
                                        const AcGePoint2dArray& pts, const AcGeDoubleArray& bulges)
{
        // see if they specified a Hatch or not
    bool doHatch = false;
    compBldr.rcpInfo().prData()->getFieldValue(_DNT("doHatch"), doHatch, false);

        // we need a temporary polyline to pass to the squiggle function
    Acad::ErrorStatus es;
    AcDbPolyline* tmpPline = new AcDbPolyline;
    int len = pts.length();
    for (int i=0; i<len; i++) {
        if (bulges.isEmpty())
            es = tmpPline->addVertexAt(i, pts[i], 0.0);
        else
            es = tmpPline->addVertexAt(i, pts[i], bulges[i]);

        AEC_ASSERT(es == Acad::eOk);
    }
    tmpPline->setClosed(true);

        // get the squiggle fudge factors from the RecipeSpec.  Retrieving them dynamically
        // from the XML file gives the user control over how smudged they want their boundary to be.
    double deltaX = compBldr.rcpInfo().rcInfo()->unitDepDefault(1.0);
    compBldr.rcpInfo().rcData()->getItem_ParamUnitDep(AecRcpIdStr::boundary, _DNT("SqDeltaX"), deltaX, false);

    double deltaY = compBldr.rcpInfo().rcInfo()->unitDepDefault(0.1);
    compBldr.rcpInfo().rcData()->getItem_ParamUnitDep(AecRcpIdStr::boundary, _DNT("SqDeltaY"), deltaY, false);

        // now squiggle the polyine we just made
    AcDbPolyline* squiggledPline = NULL;
    es = AecRcpUtils::squiggleCurve(deltaX, deltaY, tmpPline, squiggledPline); // create the squiggled pline

    delete tmpPline;
    if (es != Acad::eOk)
        return es;

    compBldr.setToDefaultProps(squiggledPline);
    compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::boundary, squiggledPline);
    
    es = compBldr.addToDb(squiggledPline);
    if (es == Acad::eOk) {
        if (doHatch) {
            AcDbHatch* hatch = NULL;
            if (compBldr.rcpInfo().rcData()->getItem_Hatch(AecRcpIdStr::infill, hatch) == Acad::eOk) {
                compBldr.setToDefaultProps(hatch);
                compBldr.rcpInfo().rcData()->getItem_EntityProps(AecRcpIdStr::infill, hatch);
                es = compBldr.addHatchToPrevBoundary(hatch, true);
            }
        }

        return compBldr.endOrAbort();     // finish our block def
    }
    else {
        return compBldr.abort();
    }
}

/****************************************************************************
**
**  SampUtils::createPlanOrElev
**      Many recipes in Plan and Elevation draw a series of horizontal lines.
**  This Util function abstracts out the common chore of creating a boundary
**  and a series of Horizontal lines and specified distances (from the mid-line
**  of the overall depth).
**
*************************************/

RcpStatus
SampUtils::createPlanOrElev(AecRecipeInfoAcc& rcpInfo, double depth,
                            const AcGeDoubleArray& offsets, const AecRmCStringArray& compIdStrs,
                            Justif justification)
{
    AEC_ASSERT(offsets.length() == compIdStrs.GetSize());

    AecEdJigEntSurfaceHlines jigEnt(depth, offsets);    // this standard JigEnt will draw the Hlines automatically for us

        // get the right default alignment (e.g., where is the "baseline"?
    if (justification == kTop)
        jigEnt.setJustification(AecEdJigEntSurface::kLeft);
    else if (justification == kCenter)
        jigEnt.setJustification(AecEdJigEntSurface::kCenter);
    else if (justification == kBottom)
        jigEnt.setJustification(AecEdJigEntSurface::kRight);

    AecEdJigSurface jig(&jigEnt);

    AecUiPr::Status stat;
    stat = jig.go();
    if (stat != AecUiPr::kOk)
        return AecRcpFunc::rcpResult(stat);

    AcGePoint2dArray pts;
    jigEnt.calculate();         // let the Jig calculate its information about how to draw the component
    jigEnt.getEcsOutline(pts);  // get the simple out line of the component

        // make a CompBldr object to help us do the drawing
    AecRcpCompBldrCurSpace compBldr(rcpInfo);
    if (compBldr.start() != Acad::eOk)
        return AecRcpFunc::rcpError();

    compBldr.pushXform(jigEnt.ecsToWcs());  // this will automatically xform any entities we push in to the correct locaiton.

        // speed things up when using a large amount of offset lines by getting template properties.
    AcDbLine* tmplLineBoundary = new AcDbLine;
    compBldr.setToDefaultProps(tmplLineBoundary);
    rcpInfo.rcData()->getItem_EntityProps(AecRcpIdStr::boundary, tmplLineBoundary);

    AcDbLine* tmplLineHidden = new AcDbLine;
    compBldr.setToDefaultProps(tmplLineHidden);
    rcpInfo.rcData()->getItem_EntityProps(AecRcpIdStr::hidden, tmplLineHidden);

        // set any properties for the boundary polyline before adding it to the Db
    AcDbPolyline* pline = new AcDbPolyline;
    pline->setPropertiesFrom(tmplLineBoundary);

    compBldr.addPolyline(pline, pts, true);    // takes responsibility for pline

    AcGePoint3dArray startPts, endPts;
    jigEnt.getOffsetLines(startPts, endPts);

        // make a line entity for each of the offset lines
    int len = startPts.length();
    for (int i=0; i<len; i++) {
        AcDbLine* tmpLine;
        tmpLine = new AcDbLine(startPts[i], endPts[i]);
        compBldr.setToDefaultProps(tmpLine);
        if ((compIdStrs[i] == _DNT("")) || (compIdStrs[i] == AecRcpIdStr::boundary))
            tmpLine->setPropertiesFrom(tmplLineBoundary);
        else if (compIdStrs[i] == AecRcpIdStr::hidden)
            tmpLine->setPropertiesFrom(tmplLineHidden);
        else
            rcpInfo.rcData()->getItem_EntityProps(compIdStrs[i], tmpLine);

        compBldr.addToDb(tmpLine);
    }

    delete tmplLineBoundary;
    delete tmplLineHidden;

    if (compBldr.endOrAbort() == Acad::eOk)
        return AecRcpFunc::rcpOk();
    else
        return AecRcpFunc::rcpError();
}