'
'  Copyright 2004-2008 by Autodesk, Inc.
'
'  Permission to use, copy, modify, and distribute this software in
'  object code form for any purpose and without fee is hereby granted, 
'  provided that the above copyright notice appears in all copies and 
'  that both that copyright notice and the limited warranty and
'  restricted rights notice below appear in all supporting 
'  documentation.
'
'  AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
'  AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'  MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
'  DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'  UNINTERRUPTED OR ERROR FREE.
'
'  Use, duplication, or disclosure by the U.S. Government is subject to 
'  restrictions set forth in FAR 52.227-19 (Commercial Computer
'  Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'  (Rights in Technical Data and Computer Software), as applicable.
' 
'  Last updated: Sept 1, 2008 by MH. Migrated to use .NET API in ACA 2009. (was COM Interop) 
' 
Option Explicit On
Option Strict On

Imports System                          ' Double
Imports System.Windows.Forms            ' MessageBox

Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Recipe
Imports Autodesk.Aec.Recipe.Utilities
Imports Autodesk.Aec.Arch.DatabaseServices

Imports AcDb2 = Autodesk.AutoCAD.DatabaseServices

Imports TransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports DtlUtils = Autodesk.Aec.Recipe.Utilities


Public Module Module1

    Public Sub DrawRectangle(ByVal acadObj As Object)

        Dim rcpInfo As RecipeInformationAccess = New RecipeInformationAccess()

        ' (1) Get the raw dimensions from the database (.mdb) 
        '
        Dim x As Double = DtlUtils.Database.GetFieldValueDoubleUnitDependent("SIZE_X")
        Dim y As Double = DtlUtils.Database.GetFieldValueDoubleUnitDependent("SIZE_Y")

        ' (2) Get user input from the palette
        '
        Dim strJig As String = DtlUtils.Palette.GetPropertyValueString("jig")
        Dim bHatch As Boolean = DtlUtils.Palette.GetPropertyValueBool("hatch")

        ' (3) Get some parameters from the recipe spec
        ' MH:  Recipe here means Recipe Spec (.xml) file. 
        ' 
        Dim strPosition As String = DtlUtils.Recipe.GetDataValueString("label", "Position")
        Dim dHeightFactor As Double = DtlUtils.Recipe.GetDataValueDouble("label", "HeightFactor")

        ' (4) build a component, using compBuilder. It will take care of many house keeping tasks. 
        ' 
        Dim compBuilder As ComponentBuilderBlockDefinition = New ComponentBuilderBlockDefinition(rcpInfo)
        compBuilder.Start()

        Try
            MakeRectangle(rcpInfo, compBuilder, x, y, bHatch, dHeightFactor, strPosition)
        Catch ex As Exception
            compBuilder.Abort()
        Finally
            compBuilder.EndOrAbort()
        End Try

        ' (5) start the jig interaction. 
        ' 
        If strJig = "jigStamp" Then
            Recipe.ReturnStatus = JigUtilities.StampIt(compBuilder)

        ElseIf strJig = "jigArray" Then
            Recipe.ReturnStatus = JigUtilities.LinearArrayIt(compBuilder)

        ElseIf strJig = "jigBookends" Then

            ' we need a bit more work for bookends jig, creating additional components at the both ends. 
            ' 
            Dim compBuilderStart As ComponentBuilderBlockDefinition = New ComponentBuilderBlockDefinition(rcpInfo)
            compBuilderStart.Start()
            Dim compBuilderEnd As ComponentBuilderBlockDefinition = New ComponentBuilderBlockDefinition(rcpInfo)
            compBuilderEnd.Start()

            Try
                MakeRectangleBookends(rcpInfo, compBuilderStart, compBuilderEnd, x, y, bHatch)
            Catch ex As Exception
                compBuilderStart.Abort()
                compBuilderEnd.Abort()
            Finally
                compBuilderStart.EndOrAbort()
                compBuilderEnd.EndOrAbort()
            End Try

            ' create three block refs: at the both end and in between.
            Dim middleBlock As AcDb2.BlockReference = compBuilder.GetNewReference()
            Dim startBlock As AcDb2.BlockReference = compBuilderStart.GetNewReference()
            Dim endBlock As AcDb2.BlockReference = compBuilderEnd.GetNewReference()

            ' create another compBuilder to use with bookends jig. 
            Dim builder As ComponentBuilderCurrentSpace = New ComponentBuilderCurrentSpace(rcpInfo)

            ' now insert the blocks that we made. 
            Recipe.ReturnStatus = JigUtilities.Bookends(builder, startBlock, endBlock, middleBlock, False)

        End If

    End Sub

    'define a middle rectangle 
    '
    Public Sub MakeRectangle(ByVal rcpInfo As RecipeInformationAccess, ByRef compBuilder As ComponentBuilderBlockDefinition, ByVal x As Double, ByVal y As Double, ByVal bHatch As Boolean, ByVal dHeightFactor As Double, ByVal strPosition As String)

        ' Create the polyline
        ' 
        Dim pts As Point2dCollection = New Point2dCollection()
        pts.Add(New Point2d(0.0, 0.0))
        pts.Add(New Point2d(x, 0.0))
        pts.Add(New Point2d(x, y))
        pts.Add(New Point2d(0.0, y))

        Dim poly As Polyline = New Polyline()
        compBuilder.SetToDefaultProperties(poly)
        compBuilder.AddPolyline(poly, pts, True)

        ' Hatch, if necessary
        ' 
        If (bHatch) Then
            Dim hat As Hatch = New Hatch()
            hat.SetHatchPattern(HatchPatternType.PreDefined, "ANSI31")
            compBuilder.AddHatchToPreviousBoundary(hat, True)
        End If

        ' Add text
        ' 
        Dim pnt(2) As Double : pnt(0) = x / 2 : pnt(2) = 0

        Dim align As AttachmentPoint
        If strPosition = "Top" Then
            pnt(1) = y
            align = AttachmentPoint.BottomCenter
        ElseIf strPosition = "Bottom" Then
            pnt(1) = -y * dHeightFactor / 5
            align = AttachmentPoint.TopCenter
        Else
            pnt(1) = y / 2
            align = AttachmentPoint.MiddleCenter
        End If

        Dim strText As String = CStr(x) & "x" & CStr(y)
        Dim txt As MText = New MText
        txt.Contents = strText
        txt.Location = New Point3d(pnt)
        txt.TextHeight = y * dHeightFactor
        txt.Attachment = align
        compBuilder.AddToDatabase(txt)

    End Sub


    '  defines rectangles at both ends for bookends jig.  
    '
    Public Sub MakeRectangleBookends(ByVal rcpInfo As RecipeInformationAccess, _
                                     ByRef compBuilderStart As ComponentBuilderBlockDefinition, ByRef compBuilderEnd As ComponentBuilderBlockDefinition, _
                                     ByVal x As Double, ByVal y As Double, _
                                     ByVal bHatch As Boolean)

        ' (0) Define the bookend geometry
        '
        '          cutline1
        '       _____ /_____
        '      |     /      |
        ' left |    /__     | start (closed plines)
        '      |       /    |
        '      |_____ /_____|
        '            /
        '          cutline2
        '       (just the tips)
        '

        Dim right As Point2dCollection = New Point2dCollection()
        right.Add(New Point2d(x / 2.0, 0.0))
        right.Add(New Point2d(x, 0.0))
        right.Add(New Point2d(x, y))
        right.Add(New Point2d(x / 2.0, y))
        right.Add(New Point2d(x / 4.0, y / 2.0))
        right.Add(New Point2d(3.0 * x / 4.0, y / 2.0))
        right.Add(New Point2d(x / 2.0, 0.0))

        Dim left As Point2dCollection = New Point2dCollection()
        left.Add(New Point2d(x / 2.0, 0.0))
        left.Add(New Point2d(0, 0.0))
        left.Add(New Point2d(0, y))
        left.Add(New Point2d(x / 2.0, y))
        left.Add(New Point2d(x / 4.0, y / 2.0))
        left.Add(New Point2d(3.0 * x / 4.0, y / 2.0))
        left.Add(New Point2d(x / 2.0, 0.0))

        Dim cutline1 As Point2dCollection = New Point2dCollection()
        cutline1.Add(New Point2d(x / 4.0, -y / 2.0))
        cutline1.Add(New Point2d(3 * x / 4.0, y / 2.0))

        Dim cutline2 As Point2dCollection = New Point2dCollection()
        cutline2.Add(New Point2d(x / 4.0, y / 2.0))
        cutline2.Add(New Point2d(3 * x / 4.0, 3 * y / 2.0))


        ' (1) Create the start bookend
        '         /_____
        '        /      |
        '       /__     |
        '          /    |
        '         /_____|
        '        /

        Dim polyCutline1 As Polyline = New Polyline()
        compBuilderStart.SetToDefaultProperties(polyCutline1)
        compBuilderStart.AddPolyline(polyCutline1, cutline1, True)

        Dim polyCutline2 As Polyline = New Polyline()
        compBuilderStart.SetToDefaultProperties(polyCutline2)
        compBuilderStart.AddPolyline(polyCutline2, cutline2, True)

        Dim polyRight As Polyline = New Polyline()
        compBuilderStart.SetToDefaultProperties(polyRight)
        compBuilderStart.AddPolyline(polyRight, right, True)

        ' Hatch, if needed 
        ' 
        If (bHatch) Then
            Dim hatStart As Hatch = New Hatch()
            hatStart.SetHatchPattern(HatchPatternType.PreDefined, "ANSI31")
            compBuilderStart.AddHatchToPreviousBoundary(hatStart, True)
        End If


        ' (2) Create the end bookend
        '   _____ /
        '  |     /
        '  |    /__
        '  |       /
        '  |_____ /
        '        /

        Dim polyCutline11 As Polyline = New Polyline()
        compBuilderEnd.SetToDefaultProperties(polyCutline11)
        compBuilderEnd.AddPolyline(polyCutline11, cutline1, True)

        Dim polyCutline22 As Polyline = New Polyline()
        compBuilderEnd.SetToDefaultProperties(polyCutline22)
        compBuilderEnd.AddPolyline(polyCutline22, cutline2, True)

        Dim polyLeft As Polyline = New Polyline()
        compBuilderEnd.SetToDefaultProperties(polyLeft)
        compBuilderEnd.AddPolyline(polyLeft, left, True)

        If (bHatch) Then
            Dim hatEnd As Hatch = New Hatch()
            hatEnd.SetHatchPattern(HatchPatternType.PreDefined, "ANSI31")
            compBuilderEnd.AddHatchToPreviousBoundary(hatEnd, True)
        End If

    End Sub

End Module
